<?php

namespace App\Http\Controllers\Accounts;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use Illuminate\Http\Request;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Account\Voucher;
use App\Models\Account\VoucherAccDetails;
use App\Models\Account\VoucherDesc;
use App\Models\Bill\Bill;
use App\Models\Bill\CreditNoteDesc;
use App\Models\Booking\BookGr;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\FleetOwner\LorryHire;
use App\Models\GoodsMove\Challan;
use App\Models\Inventory\Purchase;
use App\Models\Master\IssueBooks;
use App\Models\Master\Party;
use App\Models\Master\PartyOpeningDetails;
use App\Models\Master\Vehicle;
use App\Models\Pod\Pod;
use Illuminate\Support\Facades\DB;

use Carbon\Carbon;

class VoucherController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Voucher');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $voucher = Voucher::with(['branch:id,branch_name','voucher_acc_details:id,voucher_id,account_id,nature,against,dr_amount,cr_amount', 'voucher_descs:id,voucher_id,v_acc_det_id,against_id,against,against_number,date,due_amt,receipt_amt,received_amt,tds_percent,tds,deduction,balance_amt,from_opening','voucher_acc_details.account:id,account_name'])->where('fin_year',$request->current_year)->get();
        return response()->json($voucher);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $voucher = Voucher::with(['branch:id,branch_name','voucher_acc_details:id,voucher_id,account_id,nature,against,dr_amount,cr_amount', 'voucher_descs:id,voucher_id,v_acc_det_id,against_id,against,against_number,date,due_amt,receipt_amt,received_amt,tds_percent,tds,deduction,balance_amt,from_opening','voucher_acc_details.account:id,account_name'])->find($pk);
        
        return response()->json(['voucher'=>$voucher]);
    }

    public function store(Request $request){
        
        $test = $this->is_new_permitted($request->user->id, 'Voucher');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required', 
            'voucher_name' => 'required',
            'voucher_number' => 'required',
        ], [
            'branch.required' => 'Required',
            'voucher_name.required' => 'Required',
            'voucher_name.required' => 'Required',
        ]);

        if ($request->total_dr_amount != $request->total_cr_amount ) {
            return response()->json("The Debit And Credit Amount Do Not Match", 500);
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            $max_number = $this->get_max_number_from_voucher_number($request->voucher_number);

            // Check if the voucher number is already present
            $check = Voucher::where('voucher_number', $request->voucher_number)->where('voucher_name',$request->voucher_name)->first();
            if($check){
                return response()->json(['errors' => ['voucher_number' => ['Already Exists !']]],422);
            }
            
            // SAVING A new Voucher
            $voucher = new Voucher;
            $voucher->branch_id = $request->branch['id'];
            $voucher->voucher_type = $request->voucher_type;
            $voucher->fin_yr_wise = $request->fin_yr_wise;
            $voucher->prefix = $request->prefix;
            $voucher->max_number = $max_number;
            $voucher->voucher_name = $request->voucher_name;
            $voucher->voucher_number = $request->voucher_number;
            $voucher->voucher_date = $request->voucher_date;
            $voucher->voucher_time = $request->voucher_time;
            $voucher->narration = $request->narration;
            $voucher->cheque_number = $request->cheque_number;
            $voucher->cheque_date = $request->cheque_date;
            $voucher->total_dr_amount = $request->total_dr_amount;
            $voucher->total_cr_amount = $request->total_cr_amount;
            $voucher->fin_year = $request->current_year;

            $voucher->created_by = $request->user->id;
            $voucher->save(); 
            // new Voucher SAVED

            $voucher_accounting_details = $request->voucher_accounting_details;

            foreach ($voucher_accounting_details as $index => $vad) {
                if (!$vad['account']) {
                    continue;
                }
                
                $v_acc_detail = new VoucherAccDetails();
                $v_acc_detail->voucher_id = $voucher->id;
                $v_acc_detail->voucher_name = $voucher->voucher_name;
                $v_acc_detail->voucher_number = $voucher->voucher_number;
                $v_acc_detail->account_id = $vad['account']['id'];
                $v_acc_detail->nature = $vad['nature'];
                $v_acc_detail->against = $vad['against'];
                $v_acc_detail->dr_amount = $vad['dr_amount'];
                $v_acc_detail->cr_amount = $vad['cr_amount'];
                $v_acc_detail->fin_year = $voucher->fin_year;
                $v_acc_detail->save();

                if ($vad['against']) {
                    $voucher_descs = $request->voucher_descs;
                    foreach ($voucher_descs as $vd) {
                        if (
                            !$vd['against'] || !$vd['against_number'] ||
                            ($vd['v_acc_det_id'] != $vad['id'])
                        ) {
                            continue;
                        }
                        
                        $vou_desc = new VoucherDesc();
                        $vou_desc->voucher_id = $voucher->id;
                        $vou_desc->voucher_name = $voucher->voucher_name;
                        $vou_desc->voucher_number = $voucher->voucher_number;
                        $vou_desc->v_acc_det_id = $v_acc_detail->id;
                        $vou_desc->against = $vd['against'];
                        $vou_desc->against_id = $vd['against_id'];
                        $vou_desc->against_number = $vd['against_number'];
                        $vou_desc->date = $vd['date'];
                        $vou_desc->due_amt = $vd['due_amt'];
                        $vou_desc->received_amt = $vd['received_amt'];
                        $vou_desc->receipt_amt = $vd['receipt_amt'];
                        $vou_desc->tds_percent = $vd['tds_percent'];
                        $vou_desc->tds = $vd['tds'];
                        $vou_desc->deduction = $vd['deduction'];
                        $vou_desc->balance_amt = $vd['balance_amt'];
                        $vou_desc->from_opening = $vd['from_opening'];
                        $vou_desc->fin_year = $voucher->fin_year;
                        $vou_desc->save();

                        if($vou_desc->balance_amt<=0){
                            $this->set_paid_status($vou_desc->against, $vou_desc->against_number, $vd['from_opening'], $vd['against_id'],true);
                        }

                    }
                }
            }
            // DESCS SAVED


            $newVal = $this->get_data_for_obj($voucher->id);
            $this->createEditLog('created', 'vouchers', $voucher->id, $voucher->voucher_name.' '.$voucher->voucher_number , null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'Voucher');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $voucher = Voucher::find($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($voucher->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        if ($request->total_dr_amount != $request->total_cr_amount ) {
            return response()->json("The Debit And Credit Amount Do Not Match", 500);
        }
          

        try {
            // FETCH OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            // TRANSACTION BEGINS
            DB::beginTransaction();

            // SAVING A Voucher
            $voucher->voucher_date = $request->voucher_date;
            $voucher->voucher_time = $request->voucher_time;
            $voucher->narration = $request->narration;
            $voucher->cheque_number = $request->cheque_number;
            $voucher->cheque_date = $request->cheque_date;
            $voucher->total_dr_amount = $request->total_dr_amount;
            $voucher->total_cr_amount = $request->total_cr_amount;

            $voucher->updated_by = $request->user->id;
            $voucher->save();
            // Voucher SAVED
            
            // Loop through all the desc
            $v_desc_ids = VoucherDesc::where('voucher_id',$pk)->where('fin_year',$request->current_year)->pluck('id')->toArray();
            // DELETING VOUCHER DESC
            foreach ($v_desc_ids as $cdi) {
                $voucher_desc = VoucherDesc::find($cdi);

                $this->set_paid_status($voucher_desc->against, $voucher_desc->against_number, $voucher_desc->from_opening, $voucher_desc->against_id,false);

                $voucher_desc->delete();
            }


            $v_acc_detail_deleted = VoucherAccDetails::where('voucher_id',$pk)->where('fin_year',$request->current_year)->delete();
            
            if($v_acc_detail_deleted){
                $voucher_accounting_details = $request->voucher_accounting_details;

                    foreach ($voucher_accounting_details as $index => $vad) {
                        if (!$vad['account']) {
                            continue;
                        }
                        
                        $v_acc_detail = new VoucherAccDetails();
                        $v_acc_detail->voucher_id = $voucher->id;
                        $v_acc_detail->voucher_name = $voucher->voucher_name;
                        $v_acc_detail->voucher_number = $voucher->voucher_number;
                        $v_acc_detail->account_id = $vad['account']['id'];
                        $v_acc_detail->nature = $vad['nature'];
                        $v_acc_detail->against = $vad['against'];
                        $v_acc_detail->dr_amount = $vad['dr_amount'];
                        $v_acc_detail->cr_amount = $vad['cr_amount'];
                        $v_acc_detail->fin_year = $voucher->fin_year;
                        $v_acc_detail->save();

                        if ($vad['against']) {
                            $voucher_descs = $request->voucher_descs;
                            foreach ($voucher_descs as $vd) {
                                if (
                                    !$vd['against'] || !$vd['against_number'] ||
                                    ($vd['v_acc_det_id'] != $vad['id'])
                                ) {
                                    continue;
                                }
                                
                                $vou_desc = new VoucherDesc();
                                $vou_desc->voucher_id = $voucher->id;
                                $vou_desc->voucher_name = $voucher->voucher_name;
                                $vou_desc->voucher_number = $voucher->voucher_number;
                                $vou_desc->v_acc_det_id = $v_acc_detail->id;
                                $vou_desc->against = $vd['against'];
                                $vou_desc->against_id = $vd['against_id'];
                                $vou_desc->against_number = $vd['against_number'];
                                $vou_desc->date = $vd['date'];
                                $vou_desc->due_amt = $vd['due_amt'];
                                $vou_desc->received_amt = $vd['received_amt'];
                                $vou_desc->receipt_amt = $vd['receipt_amt'];
                                $vou_desc->tds_percent = $vd['tds_percent'];
                                $vou_desc->tds = $vd['tds'];
                                $vou_desc->deduction = $vd['deduction'];
                                $vou_desc->balance_amt = $vd['balance_amt'];
                                $vou_desc->from_opening = $vd['from_opening'];
                                $vou_desc->fin_year = $voucher->fin_year;
                                $vou_desc->save();

                                if($vou_desc->balance_amt<=0){
                                    $this->set_paid_status($vou_desc->against, $vou_desc->against_number,$vd['from_opening'], $vd['against_id'], true);
                                }
                            }
                        }
                    }
                    // DESCS SAVED
            }

            $newVal = $this->get_data_for_obj($voucher->id);
            $this->createEditLog('updated', 'vouchers', $voucher->id, $voucher->voucher_name.' '.$voucher->voucher_number, $oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'Voucher');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            

        
        $voucher = Voucher::find($pk);
        
        // FETCH OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($voucher->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }
        // Loop through all the desc
        $v_desc_ids = VoucherDesc::where('voucher_id',$pk)->where('fin_year',$request->current_year)->pluck('id')->toArray();
        // DELETING VOUCHER DESC
        foreach ($v_desc_ids as $cdi) {
            $voucher_desc = VoucherDesc::find($cdi);

            $this->set_paid_status($voucher_desc->against, $voucher_desc->against_number,$voucher_desc->from_opening, $voucher_desc->against_id, false);

            $voucher_desc->delete();
        }


        $v_acc_detail_deleted = VoucherAccDetails::where('voucher_id',$pk)->where('fin_year',$request->current_year)->delete();

        //FIND AND DELETE THE Voucher
        $voucher->delete();

        $this->createEditLog('deleted', 'vouchers', $voucher->id,  $voucher->voucher_number,$oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // WHILE SEARCHING THIS WILL RUN
        $query = Voucher::select('id', 'voucher_name', 'voucher_number')->where('fin_year',$request->current_year);

        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('voucher_name',$request->voucher_name)->where('voucher_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $voucher = $query->get();

        return response(['voucher' => $voucher, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Voucher');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Voucher::select('id', 'branch_id', 'voucher_name', 'voucher_number', 'voucher_date', 'total_dr_amount', 'total_cr_amount')->with(['branch:id,branch_name'])->where('fin_year',$request->current_year)->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('voucher_number', 'like', $searchValue . '%')
                    ->orWhere('voucher_name', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $whereFilters = $request->whereFilter;

        $query = Voucher::select('id', 'voucher_name', 'voucher_number');
        
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        $exists = $query->exists();

        if($exists){
            return response()->json(['errors' => [ 'voucher_number'  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    } 


    public function get_voucher_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = Voucher::select('id','branch_id','total_dr_amount','total_cr_amount','narration','voucher_number','voucher_date','voucher_name','created_by')
        ->with(['branch:id,branch_name,branch_code','created_by:id,username', 'voucher_acc_details:id,voucher_id,account_id,nature', 'voucher_acc_details.account:id,account_name'])
        ->where('fin_year',$request->current_year);


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('voucher_date','>=',$report_data['from_date'])->where('voucher_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['branch_id']) ){
            $query->where('branch_id', $report_data['branch_id']);
        }

        if(isset($report_data['user_id']) ){
            $query->where('created_by', $report_data['user_id']);
        }

        if (isset($report_data['party_id'])) {
            $query->whereHas('voucher_acc_details', function ($qr) use ($report_data) {
                $qr->where('account_id', $report_data['party_id']);
            });
        }
        
        if(isset($report_data['voucher_name']) ){
            $query->where('voucher_name', $report_data['voucher_name']);
        }

        // if ($searchValue) {
        //     $query->where(function ($query) use ($searchValue) {
        //         $query->where('voucher_number', 'like', '%'. $searchValue . '%');
        //     });
        // }

        //$data = $query->orderBy($column, $dir)->paginate($length);

        $data = $query->orderBy($column, $dir)->get();


        foreach ($data as $value) {
                $result_data[] = [
                    'id' => $value->id,
                    'branch' => $value->branch->branch_name,
                    'voucher_number' => $value->voucher_number,
                    'voucher_date' => $value->voucher_date,
                    'voucher_name' => $value->voucher_name,
                    'narration' => $value->narration,
                    'total_dr_amount' => $value->total_dr_amount,
                    'total_cr_amount' => $value->total_cr_amount,
                    'created_by' => $value->created_by,
                    'voucher_acc_details' => $value->voucher_acc_details,
                ] ;
            
        }

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function get_next_voucher_number(Request $request){
        $type=$request->type;
        $voucher_name = $request->voucher_name;
        $prefix=$request->prefix;
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;

        $fin_yr_wise = $request->voucher_number_fin_yr_wise;

        $year_stamp = $this->get_short_name($request->current_year);

        if($type=='SYSTEM'){
            $max = Voucher::where('branch_id',$branch_id)->where('voucher_type', $type)->where('voucher_name',$voucher_name)->where('fin_yr_wise',$fin_yr_wise)->max('max_number');
        
            if($fin_yr_wise){
                $voucher_number = $branch_code.'/'.$year_stamp.'/'.sprintf('%06d', $max+1);
            }
            else{
                $voucher_number = $branch_code.'-'.sprintf('%06d', $max+1);
            }
            return response()->json(['voucher_number' => strtoupper($voucher_number), 'max_number' => $max+1]);
        }
        else if($type=='PRINTED'){
            // GET THE NEXT NUMBER THAT HAS BEEN ISSUE TO THE BRANCH WITH THE CERTAIN PREFIX
            $max = Voucher::where('prefix',$prefix)->where('voucher_type', $type)->where('voucher_name',$voucher_name)->where('branch_id', $branch_id)->max('max_number');
            
            // IF MAX IS PRESENT, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            if( $max ){
                $check = IssueBooks::select('prefix','party_id')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type',$voucher_name)->where('issue_from', '<=', $max+1)->where('issue_to', '>=', $max+1)->first();
                if($max && $check){
                    $voucher_number = $prefix ? $prefix.'-'.($max+1) : ($max+1);
                    return response()->json(['voucher_number' => strtoupper($voucher_number), 'max_number' => $max+1, 'party' => $check->party]);
                }
            }

            // IF MAX IS NOT PRESENT OR NOT VALID, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            $check = IssueBooks::select('prefix','party_id','issue_from')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type',$voucher_name)->where('issue_from', '>=', $max+1)->first();
            if( $check ){
                $voucher_number = $prefix ? $prefix.'-'.$check->issue_from : $check->issue_from;
                return response()->json(['voucher_number' => strtoupper($voucher_number), 'max_number' => $check->issue_from, 'party' => $check->party]);
            }
            else{
                return response()->json(['errors' => ['voucher_number' => ['No Valid Voucher Number']]],422);
            }
        }
    }

    public function autocomplete_for_against(Request $request, $selection){
        $searchVariable = $request->searchVariable;
        $acc_party_id = $request->acc_party_id;
        $notInFilters = isset($request->filterby['notInFilters']) ? $request->filterby['notInFilters'] : [];
        $voucher_name = $request->voucher_name;
        
        $soft_deleted_voucher_descs = isset($request->soft_deleted_voucher_descs)?$request->soft_deleted_voucher_descs['against_numbers']:[];

        $concate_val = $voucher_name == 'Receipt Voucher' ? 'dr' : 'cr';

        $concatinate_openings = [];

        $commonConditions = function ($query) use ($soft_deleted_voucher_descs) {
            $query->where('paid_status', 0)
                ->orWhereIn('against_number', $soft_deleted_voucher_descs);
        };

        if($selection == 'Purchase'){
            $against_query = Purchase::select('id','bill_number as against_number')
                ->where('bill_number', 'like', '%'. $searchVariable . '%')
                ->where('acc_party_id', $acc_party_id)
                ->where($commonConditions);
            
            // Get Party Id From $acc_party_id
            $party_id = Party::select('id')->where('account_id',$acc_party_id)->first()->id;
            $concatinate_openings = PartyOpeningDetails::select('id as opening_id','against_number')->where('party_id',$party_id)->where('against', 'Purchase')->where('against_number', 'like', '%'. $searchVariable . '%')->where($concate_val,'>',0)->where($commonConditions)->get();      
        }  


        if($selection == 'Bill'){
            $against_query = Bill::select('id','bill_number as against_number')
                ->where('bill_number', 'like', '%'. $searchVariable . '%')
                ->where('acc_party_id', $acc_party_id)
                ->where($commonConditions);
                // ->whereHas('cn_desc', function ($subquery) {
                //     $subquery->whereRaw('bills.total_without_gst + bills.other_plus - bills.other_minus > credit_note_descs.amount');
                // });

                $against_query->when(
                    $against_query->getModel()->relationLoaded('cn_desc'),
                    function ($query) {
                        return $query->whereHas('cn_desc', function ($subquery) {
                            $subquery->whereRaw('bills.total_without_gst + bills.other_plus - bills.other_minus > credit_note_descs.amount');
                        });
                    }
                );
                
                

            // Get Party Id From $acc_party_id
            $party_id = Party::select('id')->where('account_id',$acc_party_id)->first()->id;
            $concatinate_openings = PartyOpeningDetails::select('id as opening_id','against_number')->where('party_id',$party_id)->where('against', 'Bill')->where('against_number', 'like', '%'. $searchVariable . '%')->where($concate_val,'>',0)->where($commonConditions)->get(); 
        }  

        if($selection == 'GR'){
            $against_query = BookGr::select('id','gr_number as against_number')
                ->where('gr_number', 'like', '%'. $searchVariable . '%')
                ->where('acc_party_id', $acc_party_id)
                ->whereHas('acc_party', function ($query) {
                    $query->where('account_type', 'Party');
                })
                ->where($commonConditions);

            // Get Party Id From $acc_party_id
            $party_id = Party::select('id')->where('account_id',$acc_party_id)->first()->id;
            $concatinate_openings = PartyOpeningDetails::select('id as opening_id','against_number')->where('party_id',$party_id)->where('against', 'GR')->where('against_number', 'like', '%'. $searchVariable . '%')->where($concate_val,'>',0)->where($commonConditions)->get();  
        }  


        if($selection == 'Loading Slip'){
            $against_query = LoadingSlip::select('id','ls_number as against_number')
                ->where('ls_number', 'like', '%'. $searchVariable . '%')
                ->where('acc_party_id', $acc_party_id)
                ->whereHas('acc_party', function ($query) {
                    $query->where('account_type', 'Party');
                })
                ->where($commonConditions);
            
            // Get Party Id From $acc_party_id
            $party_id = Party::select('id')->where('account_id',$acc_party_id)->first()->id;
            $concatinate_openings = PartyOpeningDetails::select('id as opening_id','against_number')->where('party_id',$party_id)->where('against', 'Loading Slip')->where('against_number', 'like', '%'. $searchVariable . '%')->where($concate_val,'>',0)->where($commonConditions)->get(); 
        } 

        if($selection == 'POD'){
            $against_query = Pod::select('id','pod_number as against_number')
                ->where('pod_number', 'like', '%'. $searchVariable . '%')
                ->where('acc_party_id', $acc_party_id)
                ->whereHas('acc_party', function ($query) {
                    $query->where('account_type', 'Party');
                })
                ->where($commonConditions);
        }

        if($selection == 'Challan'){
            $against_query = Challan::select('id','challan_number as against_number')
                ->where('challan_number', 'like', '%'. $searchVariable . '%')
                ->where('lorry_hire_from', $acc_party_id)
                ->whereHas('lorry_hire_from', function ($query) {
                    $query->where('account_type', 'Party');
                })
                ->where($commonConditions);
                
            // Get Party Id From $acc_party_id
            $party_id = Party::select('id')->where('account_id',$acc_party_id)->first()->id;
            $concatinate_openings = PartyOpeningDetails::select('id as opening_id','against_number')->where('party_id',$party_id)->where('against', 'Challan')->where('against_number', 'like', '%'. $searchVariable . '%')->where($concate_val,'>',0)->where($commonConditions)->get(); 
        }  

        if($selection == 'Lorry Hire'){
            $against_query = LorryHire::select('id','lh_number as against_number')
                ->where('lh_number', 'like', '%'. $searchVariable . '%')
                ->where('lorry_hire_from', $acc_party_id)
                ->whereHas('lorry_hire_from', function ($query) {
                    $query->where('account_type', 'Party');
                })
                ->where($commonConditions);
                
            // Get Party Id From $acc_party_id
            $party_id = Party::select('id')->where('account_id',$acc_party_id)->first()->id;
            $concatinate_openings = PartyOpeningDetails::select('id as opening_id','against_number')->where('party_id',$party_id)->where('against', 'Lorry Hire')->where('against_number', 'like', '%'. $searchVariable . '%')->where($concate_val,'>',0)->where($commonConditions)->get(); 
        }  

        if($selection == 'GR_COST_CENTER'){
            $against_query = BookGr::select('id','gr_number as against_number','gr_date as date')
                ->where('gr_number', 'like', '%'. $searchVariable . '%')
                ->whereDoesntHave('bill_desc');
            
            // Get Party Id From $acc_party_id
            $party_id = Party::select('id')->where('account_id',$acc_party_id)->first(); 
            if($party_id){
                $party_id = $party_id->id;
                $concatinate_openings = PartyOpeningDetails::select('id as opening_id','against_number','date')->where('party_id',$party_id)->where('against', 'GR')->where('against_number', 'like', '%'. $searchVariable . '%')->where($concate_val,'>',0)->where($commonConditions)->get(); 
            }
        }
        
        if($selection == 'LS_COST_CENTER'){
            $against_query = LoadingSlip::select('id','ls_number as against_number','ls_date as date')
                ->where('ls_number', 'like', '%'. $searchVariable . '%')
                ->whereDoesntHave('bill_desc');

            // Get Party Id From $acc_party_id
            $party_id = Party::select('id')->where('account_id',$acc_party_id)->first();
            if($party_id){
                $party_id = $party_id->id;
                $concatinate_openings = PartyOpeningDetails::select('id as opening_id','against_number','date')->where('party_id',$party_id)->where('against', 'Loading Slip')->where('against_number', 'like', '%'. $searchVariable . '%')->where($commonConditions)->get(); 
            }
        }

        if($selection == 'CHALLAN_COST_CENTER'){
            $against_query = Challan::select('id','challan_number as against_number','challan_date as date')
                ->where('challan_number', 'like', '%'. $searchVariable . '%');
        }

        if($selection == 'VEHICLE_COST_CENTER'){
            $against_query = Vehicle::select('id','vehicle_number as against_number')
                ->where('vehicle_number', 'like', '%'. $searchVariable . '%');
        }

        foreach ($notInFilters as $notInFilter) {
            if(isset($notInFilter['key']) && isset($notInFilter['values']))
            $against_query->whereNotIn($notInFilter['key'], $notInFilter['values']);
        }

        // Retrieve the filtered 
        $against = $against_query->take($request->limit)->get();

        // Concatinate the $concatinate_openings to $against
        if( count($concatinate_openings)>0 ){
            $against = $against->union($concatinate_openings);
        }
        
  
        return response(['against_numbers' => $against, 'draw' => $request->draw]);
    }

    public function fetch_against_details(Request $request, $selection, $pk){
        
        $soft_deleted_voucher_descs = isset($request->soft_deleted_voucher_descs) ? $request->soft_deleted_voucher_descs['vd_ids'] : [];
        
        $from_opening = $request->from_opening;

        $voucher_name = $request->voucher_name;
        $concate_val = $voucher_name == 'Receipt Voucher' ? 'dr' : 'cr';

        $response = [];

        if($from_opening){
            $party_op_details = PartyOpeningDetails::select('id','against_number','date',$concate_val)->find($pk);
            $balance = $this->check_for_balance($party_op_details, $party_op_details[$concate_val], $selection, $soft_deleted_voucher_descs );
            if($balance>0){
                $response = ['id' => $party_op_details->id, 'against_number'=>$party_op_details->against_number, 'date'=>$party_op_details->date, 'balance'=>round($balance, 2), 'taxable_amt'=>round($balance, 2), 'from_opening'=>true];
            }
        }
        else{
            if($selection == 'Purchase'){
                $bill = Purchase::select('id','bill_number','bill_date','grand_total_amount','total_without_gst','total_without_rounding')->find($pk);
                
                $balance = $this->check_for_balance($bill, $bill->total_without_rounding,$selection, $soft_deleted_voucher_descs );
                

                if($balance>0){
                    $response = ['id' => $bill->id, 'against_number'=>$bill->bill_number, 'date'=>$bill->bill_date, 'balance'=>round($balance, 2), 'taxable_amt'=>$bill->total_without_gst, 'from_opening'=>false];
                }
            } 

            if($selection == 'Bill'){
                $bill = Bill::select('id','bill_number','bill_date','grand_total_amount','total_without_rounding','total_without_gst','other_plus','other_minus','selection_type','pay_mode')->find($pk);
                


                $balance = $this->check_for_balance($bill, $bill->total_without_rounding,$selection, $soft_deleted_voucher_descs );

                // If bill exists in credit note, substact the value
                $cn = CreditNoteDesc::where('bill_id', $bill->id)->first();
                if($cn){
                    $balance = round($balance - $cn->net_amount, 2);
                }
            
                if($balance>0){
                    $response = ['id' => $bill->id, 'against_number'=>$bill->bill_number, 'date'=>$bill->bill_date, 'balance'=>round($balance, 2),'taxable_amt'=>$bill->total_without_gst + $bill->other_plus - $bill->other_minus, 'from_opening'=>false];
                }
            } 
    
            if($selection == 'GR'){
                $gr = BookGr::select('id','gr_number','gr_date','grand_total','total_without_gst','total_without_rounding','acc_party_id','consignor_id','consignor_name')->find($pk);
                

                $balance = $this->check_for_balance($gr, $gr->total_without_rounding,$selection, $soft_deleted_voucher_descs);
            
                if($balance>0){
                    $response = ['id' => $gr->id, 'against_number'=>$gr->gr_number,'date'=>$gr->gr_date, 'balance'=>round($balance, 2),'taxable_amt'=>$gr->total_without_gst, 'from_opening'=>false, 'consignor_name'=>$gr->consignor_name];
                }
            } 
    
            if($selection == 'Loading Slip'){
                $ls = LoadingSlip::select('id','ls_number','ls_date','balance_receivable')->find($pk);
                $balance = $this->check_for_balance($ls, $ls->balance_receivable,$selection, $soft_deleted_voucher_descs);
            
                if($balance>0){
                    $response = ['id' => $ls->id, 'against_number'=>$ls->ls_number,'date'=>$ls->ls_date, 'balance'=>round($balance, 2),'taxable_amt'=>$ls->total_without_gst, 'from_opening'=>false];
                }
            } 
    
            if($selection == 'POD'){
                $pod = Pod::select('id','pod_number','delivery_date','grand_total','total_without_rounding','total_without_gst')
                           ->with([
                            'pod_desc:id,pod_id,gr_id'
                           ])
                           ->find($pk);

                $balance = $this->check_for_balance($pod, $pod->total_without_rounding,$selection, $soft_deleted_voucher_descs);
            
                if($balance>0){
                    $response = ['id' => $pod->id, 'against_number'=>$pod->pod_number, 'date'=>$pod->delivery_date, 'balance'=>round($balance, 2),'taxable_amt'=>$pod->total_without_gst, 'from_opening'=>false];
                }
            }
    
            if($selection == 'Challan'){
                $challan = Challan::select('id','challan_number','challan_date','lh_balance','lh_net_freight')->find($pk);
                $balance = $this->check_for_balance($challan, $challan->lh_balance,$selection, $soft_deleted_voucher_descs);
            
                if($balance>0){
                    $response = ['id' => $challan->id, 'against_number'=>$challan->challan_number,'date'=>$challan->challan_date, 'balance'=>round($balance, 2),'taxable_amt'=>$challan->lh_net_freight, 'from_opening'=>false];
                }
            }

            if($selection == 'Lorry Hire'){
                $lorry_hire = LorryHire::select('id','lh_number','lh_date','balance','net_freight')->find($pk);
                $balance = $this->check_for_balance($lorry_hire, $lorry_hire->balance,$selection, $soft_deleted_voucher_descs);
                
            
                if($balance>0){
                    $response = ['id' => $lorry_hire->id, 'against_number'=>$lorry_hire->lh_number,'date'=>$lorry_hire->lh_date, 'balance'=>round($balance, 2),'taxable_amt'=>$lorry_hire->net_freight, 'from_opening'=>false];
                }
            }

        }

        

        

        return response()->json($response); 

    }

    protected function get_data_for_obj($pk){
        $voucher = Voucher::with(['branch:id,branch_name','voucher_acc_details:id,voucher_id,account_id,nature,against,dr_amount,cr_amount', 'voucher_descs:id,voucher_id,v_acc_det_id,against,against_number,date,due_amt,receipt_amt,received_amt,tds_percent,tds,deduction,balance_amt,from_opening','voucher_acc_details.account:id,account_name'])->find($pk);

        return ['voucher'=>$voucher];
    }

    protected function get_max_number_from_voucher_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected function set_paid_status($against, $against_number,$from_opening,$against_id, $status){
        if($from_opening){
           $prt_op = PartyOpeningDetails::find($against_id);
           $prt_op->paid_status = $status;
           $prt_op->save();
        }
        else{
            if($against=="GR"){
                $gr = BookGr::where('gr_number',$against_number)->first();
                $gr->paid_status = $status;
                $gr->save();
            }
            else if($against=="Purchase"){
                $bill = Purchase::where('bill_number',$against_number)->first();
                $bill->paid_status = $status;
                $bill->save();
            }
            else if($against=="Bill"){
                $bill = Bill::where('bill_number',$against_number)->first();
                $bill->paid_status = $status;
                $bill->save();
            }
            else if($against=="POD"){
                $pod = Pod::where('pod_number',$against_number)->first();
                $pod->paid_status = $status;
                $pod->save();
            }
            else if($against=="Loading Slip"){
                $ls = LoadingSlip::where('ls_number',$against_number)->first();
                $ls->paid_status = $status;
                $ls->save();
            }
            else if($against=="Challan"){
                $ch = Challan::where('challan_number',$against_number)->first();
                $ch->paid_status = $status;
                $ch->save();
            }
        }
    }

    protected function check_for_balance($against,$default_balance,$selection, $soft_deleted_voucher_descs){
            $filteredVoucherDesc = $against->voucher_desc->where('against', $selection)->whereNotIn('id', $soft_deleted_voucher_descs);
            
            if ($filteredVoucherDesc->isEmpty()) {
                // No voucher_desc, return grand_total_amount as balance
                return $default_balance;
            }
            
            $recievedAmtSum = $filteredVoucherDesc->sum('receipt_amt');
            $deductionSum = $filteredVoucherDesc->sum('deduction');
            
            $balance = $default_balance - $recievedAmtSum - $deductionSum ;
            return $balance;
    }
        
}


    

