<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Models\Apis\ApiCounter;
use App\Models\Booking\BookGr;
use App\Models\Master\Party;
use Barryvdh\Snappy\Facades\SnappyPdf;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\View;

class WhatsappController extends Controller
{
    public function sender(Request $request){

        $type = $request->type;
        $company_name = $request->company['company_name'];
        $mail_to = $request->mail_to;
         
        
        if($type == 'GR' ){
            
            $book_gr = BookGr::with(['consignor:id,party_name','consignee:id,party_name','acc_party:id,account_name','source:id,station_name','destination:id,station_name'])->where('gr_number',$request->gr_number)->first();

            $gr_date = date('d-m-Y', strtotime($book_gr->gr_date));

            $subject = 'GR Number : '.$request->gr_number.'   Dated:'.$gr_date;

            $source = $book_gr->source['station_name'];
            $destination = $book_gr->destination['station_name'];
            
            $pdf = SnappyPdf::loadHTML(View::make('docs.gr_print', ['data' => $book_gr])->render());
            
            // Set paper size and orientation
            $pdf->setOption('orientation', 'portrait');
            $pdf->setOption('page-size', 'A4');

           
            // Get the PDF content
            $pdfContent = $pdf->output();

            $filename = $request->gr_number . '.pdf';

            $attachmentPath = storage_path('app/gr_attachments/' . $request->gr_number . '.pdf');
            file_put_contents($attachmentPath, $pdfContent);

            if($mail_to=='Consignor' || $mail_to=='Both'){
                $party_name = $book_gr->consignor['party_name'];

                $receiver_number = Party::select('tele_number')->where('party_name',$party_name)->value('tele_number');

                if(!$receiver_number){
                    return response(['error_message'=>'Consignor Contact Not Found']);
                }        

                $body =  "Dear *$party_name*, \n\nWe hope this message finds you well. Thank you for selecting $company_name as your logistics partner. We are pleased to inform you that your Goods Receipt (GR) request has been successfully processed. \n\nPlease find attached the PDF document containing the details of your GR: \n\nGR Number: *$request->gr_number* \nDate of GR: $gr_date \nPickup Location: $source \nDelivery Location: $destination \n\nBest regards,\n$company_name";

                $response =  $this->sendWhatsapp($receiver_number, $subject, $body, $request->company_name, true, $attachmentPath, $filename);

            }

            if($mail_to=='Consignee' || $mail_to=='Both'){
                $party_name = $book_gr->consignee['party_name'];

                $receiver_number = Party::select('tele_number')->where('party_name',$party_name)->value('tele_number');

                if(!$receiver_number){
                    return response(['error_message'=>'Consignor Contact Not Found']);
                }        

                $body =  "Dear *$party_name*, \n\nWe hope this message finds you well. Thank you for selecting $company_name as your logistics partner. We are pleased to inform you that your Goods Receipt (GR) request has been successfully processed. \n\nPlease find attached the PDF document containing the details of your GR: \n\nGR Number: *$request->gr_number* \nDate of GR: $gr_date \nPickup Location: $source \nDelivery Location: $destination \n\nBest regards,\n$company_name";

                $response =  $this->sendWhatsapp($receiver_number, $subject, $body, $request->company_name, true, $attachmentPath, $filename);

            }


        }


       return response()->json(['status'=>$response->original, 'success'=>$response->original['success']] ,$response->original['success']=="Success"?200:500);

    }

    function sendWhatsapp($toMail, $subject, $body, $company_name, $attachments = true, $attachmentPath, $filename)
    {
        //$apikey = "649BC85494254";
        $access_token = "65292433ed7b9";
        $instance_id = "65BC961AE0380";
        $BASE_URL = "https://mblaster.in/api/send?instance_id=$instance_id&access_token=$access_token&";

        $number = $this->mobileNumberFormater($toMail);

        $body = urlencode($body);

        try {

            // $send = $BASE_URL."number=$number&message=$body&media_url=$attachmentPath&type=media&filename=BTMS_WHATSAPP"; 
            $send = $BASE_URL."number=$number&message=$body&type=media&media_url=https://bharti.logisticsoftware.in/BRANCH-000002.pdf&filename=$filename"; 
            $client = new \GuzzleHttp\Client();
            $response = $client->post($send);      

            $result = json_decode($response->getBody());

            $this->increase_api_count();
            
            return response()->json(['success' => true, 'message' => 'Whatsapp sent successfully', 'data'=>$result]);
        } catch (Exception $e) {
            return response()->json(['success' => false, 'message' => 'Error sending whatsapp: ' . $e->getMessage()], 500);
        }
    }

    function mobileNumberFormater($mobile)
    {
        // Remove any non-digit characters from the input
        $mobile = preg_replace('/\D/', '', $mobile);

        // If the number already starts with "91", return it as-is
        if (substr($mobile, 0, 2) == "91") {
            return $mobile;
        }

        // Otherwise, add "91" to the beginning and return the result
        return ($mobile == '' || $mobile == '0' || $mobile == ',' || $mobile == ', ' || $mobile == ' ,0') ? "" : "91" . $mobile;
    }


    function increase_api_count(){
        $api_name = 'whatsapp_api';
    
        $today = Carbon::now()->format('Y-m-d');
            $api_count = ApiCounter::where('api_name',$api_name)->where('date',$today)->first();
        
            if($api_count){
                $api_count->api_count = $api_count->api_count+1;
                $api_count->save();
            }
            else{
                $api_count = new ApiCounter();
                $api_count->api_count = 1;
                $api_count->date = $today;
                $api_count->api_name = $api_name;
                $api_count->save();
            }
    
    }
}
