<?php

namespace App\Http\Controllers\FleetOwner;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Booking\BookGr;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\FleetOwner\LorryHire;
use App\Models\FleetOwner\LorryHireDesc;
use App\Models\GoodsMove\Challan;
use App\Models\Master\IssueBooks;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class LorryHireController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'LorryHireSlip');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $lorry_hires = LorryHire::with([
            'branch:id,branch_name,branch_code',
            'source:id,station_name',
            'destination:id,station_name',
            'vehicle:id,vehicle_number',
            'driver:id,emp_name',
            'broker:id,party_name',
            'lorry_hire_desc',
            'lorry_hire_from:id,account_name',
            'dr_on_account:id,account_name',
            'paid_using:id,account_name'
        ])->where('fin_year',$request->current_year)->get();

        $lorry_hires->lorry_hire_desc->transform(function ($desc) {
            if($desc->against == 'GR'){
                $book_gr = BookGr::select('gr_date','acc_party_id','vehicle_id','vehicle_number')->with(['vehicle:id,vehicle_number','acc_party:id,account_name'])->find($desc->against_id);
                $date = $book_gr->gr_date;
                $vehicle = $book_gr->vehicle ? $book_gr->vehicle->vehicle_number : $book_gr->vehicle_number;
                $party = $book_gr->acc_party->account_name;
            }
            elseif($desc->against == 'Loading Slip'){
                $l_slip = LoadingSlip::select('ls_date','acc_party_id','vehicle_id','vehicle_number')->with(['vehicle:id,vehicle_number','acc_party:id,account_name'])->find($desc->against_id);
                $date = $l_slip->ls_date;
                $vehicle = $l_slip->vehicle ? $l_slip->vehicle->vehicle_number : $l_slip->vehicle_number;
                $party = $l_slip->acc_party->account_name;
            }
            elseif($desc->against == 'Challan'){
                $challan = Challan::select('challan_date','vehicle_id','vehicle_number')->with(['vehicle:id,vehicle_number'])->find($desc->against_id);
                $date = $challan->challan_date;
                $vehicle = $challan->vehicle ?  $challan->vehicle->vehicle_number : $challan->vehicle_number;
                $party = null;
            }

                
                return [
                    'lorry_hire_desc_id' => $desc->id,
                    'against' => $desc->against,
                    'against_number' => $desc->against_number,
                    'against_id' => $desc->against_id,
                    'vehicle' => $vehicle,
                    'party' => $party,
                    'date' => $date,
                    'loaded_wt' => $desc->loaded_wt,
                    'loaded_pkg' => $desc->loaded_pkg
                ];
        });

        return response()->json($lorry_hires);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $lorry_hire = LorryHire::with([
            'branch:id,branch_name,branch_code',
            'source:id,station_name',
            'destination:id,station_name',
            'vehicle:id,vehicle_number',
            'driver:id,emp_name',
            'broker:id,party_name',
            'lorry_hire_desc',
            'lorry_hire_from:id,account_name',
            'dr_on_account:id,account_name',
            'paid_using:id,account_name'
        ])->find($pk);

        $lorry_hire->lorry_hire_desc->transform(function ($desc) {
            if($desc->against == 'GR'){
                $book_gr = BookGr::select('gr_date','acc_party_id','vehicle_id', 'vehicle_number')->with(['vehicle:id,vehicle_number','acc_party:id,account_name'])->find($desc->against_id);
                $date = $book_gr->gr_date;
                $vehicle = $book_gr->vehicle ? $book_gr->vehicle->vehicle_number : $book_gr->vehicle_number;
                $party = $book_gr->acc_party->account_name;
            }
            elseif($desc->against == 'Loading Slip'){
                $l_slip = LoadingSlip::select('ls_date','acc_party_id','vehicle_id', 'vehicle_number')->with(['vehicle:id,vehicle_number','acc_party:id,account_name'])->find($desc->against_id);
                $date = $l_slip->ls_date;
                $vehicle = $l_slip->vehicle ? $l_slip->vehicle->vehicle_number : $l_slip->vehicle_number;
                $party = $l_slip->acc_party->account_name;
            }
            elseif($desc->against == 'Challan'){
                $challan = Challan::select('challan_date','vehicle_id', 'vehicle_number')->with(['vehicle:id,vehicle_number'])->find($desc->against_id);
                $date = $challan->challan_date;
                $vehicle = $challan->vehicle ?  $challan->vehicle->vehicle_number : $challan->vehicle_number;
                $party = null;
            }

            
            return [
                'lorry_hire_desc_id' => $desc->id,
                'against' => $desc->against,
                'against_number' => $desc->against_number,
                'against_id' => $desc->against_id,
                'vehicle' => $vehicle,
                'party' => $party,
                'date' => $date,
                'loaded_wt' => $desc->loaded_wt,
                'loaded_pkg' => $desc->loaded_pkg
            ];
        });

        return response()->json(['lorry_hire' =>$lorry_hire]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'LorryHireSlip');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'lh_number' => 'required|unique:lorry_hires,lh_number',
            'source' => 'required',
            'destination' => 'required',
            'vehicle' => 'required'
        ], [
            'branch.required' => 'Required',
            'lh_number.required' => 'Required',
            'source.required' => 'Required',
            'destination.required' => 'Required',
            'vehicle.required' => 'Required',
            'lh_number.unique' => 'Already Exist !',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            

            // IN CASE THE CHALLAN TYPE IS 'PRINTED', WE NEED TO CHECK IF THE BOOK IS ACTUALLY ISSUED TO THE BRANCH OR NOT
            $max_number = $this->get_max_number_from_lh_number($request->lh_number);
            if($request->lh_type == 'PRINTED'){
                $check = IssueBooks::select('prefix','party_id')->where('prefix',$request->prefix)->where('branch_id', $request->branch['id'])->where('book_type','LorryHireSlip')->where('issue_from', '<=', $max_number)->where('issue_to', '>=', $max_number)->first();
                if(!$check){
                    return response()->json(['message'=>'Book Not Issued To The Branch'], 422);
                }
                
            }
            
            
            // SAVING A NEW GR
            $lorry_hire = new LorryHire();
            $lorry_hire->branch_id = $request->branch['id'];
            $lorry_hire->lh_type = $request->lh_type;
            $lorry_hire->prefix = $request->prefix;
            $lorry_hire->max_number = $max_number;
            $lorry_hire->lh_number = $request->lh_number;
            $lorry_hire->lh_date = $request->lh_date;
            $lorry_hire->lh_time = $request->lh_time;

            $lorry_hire->source_id = $request->source['id'];
            $lorry_hire->destination_id = $request->destination['id'];
            $lorry_hire->vehicle_id = $request->vehicle ? $request->vehicle['id'] : null;
            $lorry_hire->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
            $lorry_hire->driver_id = $request->driver ? $request->driver['id'] : null;
            $lorry_hire->broker_id = $request->broker ? $request->broker['id'] : null;

            $lorry_hire->lorry_hire_from = $request->lorry_hire_from ? $request->lorry_hire_from['id'] : null;
            $lorry_hire->dr_on_account = $request->dr_on_account ? $request->dr_on_account['id'] : null;
            $lorry_hire->remarks = $request->remarks;

            $lorry_hire->hired_against = $request->hired_against;

            $lorry_hire->total_pkg = $request->total_pkg;
            $lorry_hire->total_wt = $request->total_wt;
            $lorry_hire->unit = $request->unit;
            $lorry_hire->rate = $request->rate;
            $lorry_hire->freight = $request->freight;
            $lorry_hire->extra_point = $request->extra_point;
            $lorry_hire->net_freight = $request->net_freight;
            $lorry_hire->munshiana = $request->munshiana;
            $lorry_hire->advance = $request->advance;
            $lorry_hire->payment = $request->payment;
            $lorry_hire->tds_percent = $request->tds_percent;
            $lorry_hire->tds = $request->tds;
            $lorry_hire->balance = $request->balance;
            $lorry_hire->paid_using = $request->paid_using ? $request->paid_using['id'] :null;

            $lorry_hire->load_date = $request->load_date;
            $lorry_hire->loading_labour = $request->loading_labour;
            $lorry_hire->loading_detention = $request->loading_detention;
            $lorry_hire->unload_date = $request->unload_date;
            $lorry_hire->unloading_labour = $request->unloading_labour;
            $lorry_hire->unloading_detention = $request->unloading_detention;
            $lorry_hire->collection_chg = $request->collection_chg;
            $lorry_hire->other_add_label = $request->other_add_label;
            $lorry_hire->other_add = $request->other_add;
            $lorry_hire->other_minus_label = $request->other_minus_label;
            $lorry_hire->other_minus = $request->other_minus;


            $lorry_hire->fin_year = $request->current_year;

            $lorry_hire->created_by = $request->user->id;

            // Save the model to the database
            $lorry_hire->save();

            

            // SAVING PACKAGE DETAILS
            $lorry_hire_desc = $request->lorry_hire_desc;
            foreach ($lorry_hire_desc as $lhd) {
                
                $lh_dc = new LorryHireDesc();
                $lh_dc->lorry_hire_id = $lorry_hire->id;
                $lh_dc->lh_number = $lorry_hire->lh_number;
                $lh_dc->against = $lhd['against'];
                $lh_dc->against_id = $lhd['against_id'];
                $lh_dc->against_number = $lhd['against_number'];
                $lh_dc->loaded_pkg = $lhd['loaded_pkg'];
                $lh_dc->loaded_wt = $lhd['loaded_wt'];
                $lh_dc->fin_year = $request->current_year;
                $lh_dc->save();
            }
            // PACKAGE DETAILS SAVED


            // RECORD THE TRANSACTION IN ACCOUNTS
           $this->record_transaction_in_accounts($request,$lorry_hire);
            // TRANSACTION RECORDED IN ACCOUNTS

             
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($lorry_hire->id);
            $this->createEditLog('created', 'lorry_hires', $lorry_hire->id, $lorry_hire->lh_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
            
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$lorry_hire->id, 'ds_number'=>$lorry_hire->challan_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'LorryHireSlip');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $lorry_hire = LorryHire::find($pk);

        // $challan_check = $this->check_in_challan('challan_number','challan_number', 'lorry_hire_id', $pk);
        // if($challan_check['status']){
        //    $status= $challan_check['status'] ? $challan_check : false;
        //    return response()->json(['error'=>'Document in use', 'data'=>$status],500);
        // };


        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($lorry_hire->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'lh_number' => 'required',
            'source' => 'required',
            'destination' => 'required',
            'vehicle' => 'required'
        ], [
            'branch.required' => 'Required',
            'lh_number.required' => 'Required',
            'source.required' => 'Required',
            'destination.required' => 'Required',
            'vehicle.required' => 'Required',
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            // SAVING A NEW CHALLAN
            $lorry_hire->lh_date = $request->lh_date;
            $lorry_hire->lh_time = $request->lh_time;

            $lorry_hire->source_id = $request->source['id'];
            $lorry_hire->destination_id = $request->destination['id'];
            $lorry_hire->vehicle_id = $request->vehicle ? $request->vehicle['id'] : null;
            $lorry_hire->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
            $lorry_hire->driver_id = $request->driver ? $request->driver['id'] : null;
            $lorry_hire->broker_id = $request->broker ? $request->broker['id'] : null;

            $lorry_hire->lorry_hire_from = $request->lorry_hire_from ? $request->lorry_hire_from['id'] : null;
            $lorry_hire->dr_on_account = $request->dr_on_account ? $request->dr_on_account['id'] : null;
            $lorry_hire->remarks = $request->remarks;

            $lorry_hire->hired_against = $request->hired_against;

            $lorry_hire->total_pkg = $request->total_pkg;
            $lorry_hire->total_wt = $request->total_wt;
            $lorry_hire->unit = $request->unit;
            $lorry_hire->rate = $request->rate;
            $lorry_hire->freight = $request->freight;
            $lorry_hire->extra_point = $request->extra_point;
            $lorry_hire->net_freight = $request->net_freight;
            $lorry_hire->munshiana = $request->munshiana;
            $lorry_hire->advance = $request->advance;
            $lorry_hire->payment = $request->payment;
            $lorry_hire->tds_percent = $request->tds_percent;
            $lorry_hire->tds = $request->tds;
            $lorry_hire->balance = $request->balance;
            $lorry_hire->paid_using = $request->paid_using ? $request->paid_using['id'] :null;

            $lorry_hire->load_date = $request->load_date;
            $lorry_hire->loading_labour = $request->loading_labour;
            $lorry_hire->loading_detention = $request->loading_detention;
            $lorry_hire->unload_date = $request->unload_date;
            $lorry_hire->unloading_labour = $request->unloading_labour;
            $lorry_hire->unloading_detention = $request->unloading_detention;
            $lorry_hire->collection_chg = $request->collection_chg;
            $lorry_hire->other_add_label = $request->other_add_label;
            $lorry_hire->other_add = $request->other_add;
            $lorry_hire->other_minus_label = $request->other_minus_label;
            $lorry_hire->other_minus = $request->other_minus;

            $lorry_hire->updated_by = $request->user->id;

            $lorry_hire->save();
            // NEW CHALLAN SAVED
            
            // SAVING PACKAGE DETAILS
            $newly_added_lh_desc = $request->newly_added_lh_desc;
            foreach ($newly_added_lh_desc as $nald) {
                
                $lh_dc = new LorryHireDesc();
                $lh_dc->lorry_hire_id = $lorry_hire->id;
                $lh_dc->lh_number = $lorry_hire->lh_number;
                $lh_dc->against = $nald['against'];
                $lh_dc->against_id = $nald['against_id'];
                $lh_dc->against_number = $nald['against_number'];
                $lh_dc->loaded_pkg = $nald['loaded_pkg'];
                $lh_dc->loaded_wt = $nald['loaded_wt'];
                $lh_dc->fin_year = $request->current_year;
                $lh_dc->save();
            }
            // PACKAGE DETAILS SAVED

            // SAVING UPDATES ON EXISTING  PACKAGE DETAILS
            $updated_lh_desc = $request->updated_lh_desc;
            if( !empty($updated_lh_desc) ){
                foreach ($updated_lh_desc as $ucd) {
                       
                    $lh_dc = LorryHireDesc::find($ucd['id']);
                    $lh_dc->loaded_pkg = $nald['loaded_pkg'];
                    $lh_dc->loaded_wt = $nald['loaded_wt'];
                    $lh_dc->save();
                }
            }
            // UPDATES SAVED ON EXISTING PACKAGE DETAILS

            // DELETING EXISTING PACKAGE DETAILS
            $deleted_lh_desc = $request->deleted_lh_desc;
            if( !empty($deleted_lh_desc) ){
                foreach ($deleted_lh_desc as $dcd) {
                    $lhd = LorryHireDesc::find($dcd);
                    $lhd->delete();
                }
                    
            }
            //DELETED EXISTING PACKAGE DETAILS


            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request,$lorry_hire);
            // TRANSACTION RECORDED IN ACCOUNTS
            

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($lorry_hire->id);
            $this->createEditLog('updated', 'lorry_hires', $lorry_hire->id, $lorry_hire->lh_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'LorryHireSlip');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);
        
        $lorry_hire = LorryHire::find($pk);

        // $challan_check = $this->check_in_challan('challan_number','challan_number', 'lorry_hire_id', $pk);
        // if($challan_check['status']){
        //    $status= $challan_check['status'] ? $challan_check : false;
        //    return response()->json(['error'=>'Document in use', 'data'=>$status],500);
        // };

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($lorry_hire->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();


            $lorry_hire_desc_ids = LorryHireDesc::select('id')->where('lorry_hire_id', $pk)->pluck('id')->toArray();

            $lorry_hire_desc = LorryHireDesc::whereIn('id', $lorry_hire_desc_ids)->delete();
            


            //FIND AND DELETE THE BRANCH
            $lorry_hire->delete();
            // COMMISSIONS WILL BE CASCADED AUTOMATICALLY

            //DELETE THE TRANSACTION VOUCHER
            $t_data = [];
            $t_data['t_against'] = 'LorryHireSlip';
            $t_data['t_against_id'] = $lorry_hire->id;
            $t_data['t_against_no'] = $lorry_hire->lh_number;
            $t_data['fin_year'] = $request->current_year;

            $this->delete_transaction_voucher($t_data);


            $this->createEditLog('deleted', 'lorry_hires', $lorry_hire->id, $lorry_hire->lh_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = LorryHire::select('id', 'lh_number')->where('fin_year',$request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }
        
        }

        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('lh_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $lorry_hires = $query->get();
        return response(['lorry_hires' => $lorry_hires, 'draw' => $request->draw]);
    }

    public function against_for_lorry_hire(Request $request){
        // Initialize the query builder
        $selected_branches = $request->selected_branch_ids;
        $against_selection = $request->against_selection;
        $searchVariable = $request->searchVariable;

        if($against_selection['against'] == 'GR'){
            $query = BookGr::select('id', 'gr_number as against_number')
                    ->whereIn('branch_id', $selected_branches)
                    ->where('is_cancelled',false)
                    ->whereDoesntHave('lorry_hire_desc', function ($query) use ($against_selection) {
                        $query->where('against', 'GR');
                    })
                    ->where('gr_number', 'like', '%'. $searchVariable . '%');
        }
        elseif($against_selection['against'] == 'Loading Slip'){
            $query = LoadingSlip::select('id', 'ls_number as against_number')
                    ->whereIn('branch_id', $selected_branches)
                    ->whereDoesntHave('lorry_hire_desc', function ($query) use ($against_selection) {
                        $query->where('against', 'Loading Slip');
                    })
                    ->where('ls_number', 'like', '%'. $searchVariable . '%');
        }
        elseif($against_selection['against'] == 'Challan'){
            $query = Challan::select('id', 'challan_number as against_number')
                    ->whereIn('branch_id', $selected_branches)
                    ->whereDoesntHave('lorry_hire_desc', function ($query) use ($against_selection) {
                        $query->where('against', 'Challan');
                    })
                    ->where('challan_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $againsts = $query->get();

        return response(['againsts' => $againsts]);
    }

    public function fetch_against_details(Request $request){
        $against_selection = $request->against_selection;
        $against = $against_selection['against'];

        $against_id = isset($against_selection['against_number']) ? $against_selection['against_number']['id'] : null;
        $station_id = isset($against_selection['station']) ? $against_selection['station']['id'] : null;

        $selection_type = $against_selection['SelectionType'];

        $data = [];

        if($against == 'GR'){
            $against_details = BookGr::select('id','acc_party_id','gr_number','gr_date','vehicle_id','vehicle_number','total_pkg','total_material_weight')->where('is_cancelled',false)->with([
                                    'acc_party:id,account_name',
                                    'vehicle:id,vehicle_number'
            ])->where('is_cancelled',false);
            
            if($selection_type == 'Selection'){
                $against_details = $against_details->find($against_id);
                $against_details = [$against_details]; 
            }elseif($selection_type == 'STATION WISE'){
                $against_details = $against_details->where('destination_id', $station_id)->get();
            }      
            
            foreach($against_details as $ad){
                $data[] = [
                    'against_id' => $ad->id,
                    'against'=>'GR',
                    'against_number'=>$ad->gr_number,
                    'date' => $ad->gr_date, 
                    'party' => $ad->acc_party->account_name,
                    'loaded_pkg' => $ad->total_pkg,
                    'loaded_wt' => $ad->total_material_weight,
                    'vehicle' => $ad->vehicle ? $ad->vehicle->vehicle_number : $ad->vehicle_number,
                    'checked' => true
                ];
            }
        }
        elseif($against == 'Loading Slip'){
            $against_details = LoadingSlip::select('id','ls_number','ls_date','acc_party_id','vehicle_id','vehicle_number','pkg','material_weight')->with([
                                    'acc_party:id,account_name',
                                    'vehicle:id,vehicle_number'
            ]);

            if($selection_type == 'Selection'){
                $against_details = $against_details->find($against_id);
                $against_details = [$against_details]; 
            }elseif($selection_type == 'STATION WISE'){
                $against_details = $against_details->where('destination_id', $station_id)->get();
            }   

            foreach($against_details as $ad){
                $data[] = [
                    'against_id' => $ad->id,
                    'against'=>'Loading Slip',
                    'against_number'=>$ad->ls_number,
                    'date' => $ad->ls_date, 
                    'party' => $ad->acc_party->account_name,
                    'loaded_pkg' => $ad->pkg,
                    'loaded_wt' => $ad->material_weight,
                    'vehicle' => $ad->vehicle ? $ad->vehicle->vehicle_number : $ad->vehicle_number,
                    'checked' => true
                ];
            }

        }elseif($against == 'Challan'){
            $against_details = Challan::select('id','challan_number','challan_date','vehicle_id','vehicle_number','lh_total_pkg','lh_total_wt')->with([
                                    'vehicle:id,vehicle_number',
                               ]);

            if($selection_type == 'Selection'){
                $against_details = $against_details->find($against_id);
                $against_details = [$against_details]; 
            }elseif($selection_type == 'STATION WISE'){
                $against_details = $against_details->where('destination_id', $station_id)->get();
            } 
            
            
            foreach($against_details as $ad){
                $data[] = [
                    'against_id' => $ad->id,
                    'against'=>'Challan',
                    'against_number'=>$ad->challan_number,
                    'date' => $ad->challan_date, 
                    'party' => null,
                    'loaded_pkg' => $ad->lh_total_pkg,
                    'loaded_wt' => $ad->lh_total_wt,
                    'vehicle' => $ad->vehicle ? $ad->vehicle->vehicle_number : $ad->vehicle_number,
                    'checked' => true
                ];
            }
        }
    
        

        return response()->json(['against_details' => $data]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'LorryHireSlip');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = LorryHire::with([
                    'branch:id,branch_name,branch_code',
                    'source:id,station_name',
                    'destination:id,station_name',
                    'vehicle:id,vehicle_number',
                    'driver:id,emp_name',
                    'lorry_hire_desc',
                    'lorry_hire_from:id,account_name',
                    'dr_on_account:id,account_name',
                    'paid_using:id,account_name'
                ])
                ->where('fin_year',$request->current_year)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('lh_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = LorryHire::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $lorry_hiresQuery = LorryHire::select('id', 'lh_number')->where('fin_year',$request->current_year);
        
        if (!empty($request->user_branch_ids)) {
            $lorry_hiresQuery->whereIn('branch_id', $request->user_branch_ids);
        }
        
        $lorry_hires = $lorry_hiresQuery->orderBy('id', 'desc')->take(10)->get();
        
        return response()->json(['lorry_hires' => $lorry_hires]);
    }
    


    public function get_next_lh_number(Request $request){
        $type=$request->type;
        $prefix=$request->prefix;
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;

        if($type=='SYSTEM'){
            $max = LorryHire::where('branch_id',$branch_id)->where('lh_type', $type)->max('max_number');
            $lorry_hire_number = $branch_code.'-'.sprintf('%06d', $max+1);
            return response()->json(['lh_number' => strtoupper($lorry_hire_number), 'max_number' => $max+1]);
        }
        else if($type=='PRINTED'){
            // GET THE NEXT NUMBER THAT HAS BEEN ISSUE TO THE BRANCH WITH THE CERTAIN PREFIX
            $max = LorryHire::where('prefix',$prefix)->where('challan_type', $type)->where('branch_id', $branch_id)->max('max_number');
            
            // IF MAX IS PRESENT, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            if( $max ){
                $check = IssueBooks::select('prefix','party_id')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','Lorry Hire')->where('issue_from', '<=', $max+1)->where('issue_to', '>=', $max+1)->first();
                if($max && $check){
                    $lorry_hire_number = $prefix ? $prefix.'-'.($max+1) : ($max+1);
                    return response()->json(['lh_number' => strtoupper($lorry_hire_number), 'max_number' => $max+1, 'party' => $check->party]);
                }
            }

            // IF MAX IS NOT PRESENT OR NOT VALID, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            $check = IssueBooks::select('prefix','party_id','issue_from')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','Lorry Hire')->where('issue_from', '>=', $max+1)->first();
            if( $check ){
                $lorry_hire_number = $prefix ? $prefix.'-'.$check->issue_from : $check->issue_from;
                return response()->json(['lh_number' => strtoupper($lorry_hire_number), 'max_number' => $check->issue_from, 'party' => $check->party]);
            }
            else{
                return response()->json(['errors' => ['lh_number' => ['No Valid Lorry Hire Number']]],422);
            }
        }
    }

    protected function get_data_for_obj($pk){
        // FETCH AND SEND
        $lorry_hire = LorryHire::with([
            'branch:id,branch_name,branch_code',
            'source:id,station_name',
            'destination:id,station_name',
            'vehicle:id,vehicle_number',
            'driver:id,emp_name',
            'broker:id,party_name',
            'lorry_hire_desc',
            'lorry_hire_from:id,account_name',
            'dr_on_account:id,account_name',
            'paid_using:id,account_name'
        ])->find($pk);

        
            $lorry_hire->lorry_hire_desc->transform(function ($desc) {
                if($desc->against == 'GR'){
                    $book_gr = BookGr::select('gr_date','acc_party_id','vehicle_id','vehicle_number')->with(['vehicle:id,vehicle_number','acc_party:id,account_name'])->find($desc->against_id);
                    $date = $book_gr->gr_date;
                    $vehicle = $book_gr->vehicle ? $book_gr->vehicle->vehicle_number : $book_gr->vehicle_number;
                    $party = $book_gr->acc_party->account_name;
                }
                elseif($desc->against == 'Loading Slip'){
                    $l_slip = LoadingSlip::select('ls_date','acc_party_id','vehicle_id','vehicle_number')->with(['vehicle:id,vehicle_number','acc_party:id,account_name'])->find($desc->against_id);
                    $date = $l_slip->ls_date;
                    $vehicle = $l_slip->vehicle ? $l_slip->vehicle->vehicle_number : $l_slip->vehicle_number;
                    $party = $l_slip->acc_party->account_name;
                }
                elseif($desc->against == 'Challan'){
                    $challan = Challan::select('challan_date','vehicle_id','vehicle_number')->with(['vehicle:id,vehicle_number'])->find($desc->against_id);
                    $date = $challan->challan_date;
                    $vehicle = $challan->vehicle ?  $challan->vehicle->vehicle_number : $challan->vehicle_number;
                    $party = null;
                }

                
                return [
                    'lorry_hire_desc_id' => $desc->id,
                    'against' => $desc->against,
                    'against_number' => $desc->against_number,
                    'against_id' => $desc->against_id,
                    'vehicle' => $vehicle,
                    'party' => $party,
                    'date' => $date,
                    'loaded_wt' => $desc->loaded_wt,
                    'loaded_pkg' => $desc->loaded_pkg
                ];
            });

        return ['lorry_hire' =>$lorry_hire];
    }


    public function get_custom_fields(Request $request){
        $lorry_hire = LorryHire::select($request->custom_fields)->with($request->with_relations)->find($request->lh_id);
        return response()->json(['lorry_hire' => $lorry_hire]);
    }

    public function get_lorry_hire_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = LorryHire::with(['branch:id,branch_name,branch_code','source:id,station_name', 'vehicle:id,vehicle_number', 'driver:id,emp_name','destination:id,station_name','lorry_hire_desc','lorry_hire_from:id,account_name','broker:id,party_name','dr_on_account:id,account_name','paid_using:id,account_name','voucher_desc'=>function($query){
            $query->where('against','Lorry Hire')->where('voucher_name','Payment Voucher');
        }])->where('fin_year',$request->current_year);

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['lh_num_from']) && isset($report_data['lh_num_to'])){
            $query->where('lh_number','>=',$report_data['lh_num_from']['lh_number'])->where('lh_number','<=',$report_data['lh_num_to']['lh_number']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('lh_date','>=',$report_data['from_date'])->where('lh_date','<=',$report_data['to_date']);
        }


        if(isset($report_data['vehicle']) ){
            $query->where('vehicle_id', $report_data['vehicle']['id']);
        }


        if(isset($report_data['broker']) ){
            $query->where('broker_id', $report_data['broker']['id']);
        }



        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('lh_number', 'like','%'. $searchValue . '%');
            });
        }

        $data = $query->orderBy($column, $dir)->paginate($length);
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }


    protected function get_max_number_from_lh_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected  function  record_transaction_in_accounts($request,$lorry_hire){
        $t_data = [];
        $t_data['t_against'] = 'LorryHireSlip';
        $t_data['t_against_id'] = $lorry_hire->id;
        $t_data['t_against_no'] = $lorry_hire->lh_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);

        $t_data['branch'] = $request->branch;
        $t_data['voucher_date'] = $request->lh_date;
        $t_data['voucher_time'] = $request->lh_time;
        
        $t_data['t_against_narration'] = $lorry_hire->remarks;
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];
        
        $t_data['cancellable'] = 1;
        if($lorry_hire->dr_on_account && $lorry_hire->lorry_hire_from && $lorry_hire->net_freight>0){
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$lorry_hire->dr_on_account, 'nature'=>'dr' , 'dr_amount'=>$lorry_hire->net_freight, 'cr_amount'=>0 ];
            

            // CREDIT INCOME ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$lorry_hire->lorry_hire_from, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$lorry_hire->net_freight ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }

            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr;


            $this->create_transaction_voucher( $t_data );

        }

        $t_data['cancellable'] = 1;
        if($lorry_hire->lorry_hire_from && $lorry_hire->tds>0){
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$lorry_hire->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$lorry_hire->tds, 'cr_amount'=>0 ];
            
            // TDS PAYABLE
            $t_data['voucher_accounting_details'][] = ['account_id'=>4, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$lorry_hire->tds ];
            
            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }

            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr;


            $this->create_transaction_voucher( $t_data );

        }
        $t_data['cancellable'] = 0;
        if($lorry_hire->lorry_hire_from && $lorry_hire->paid_using && $lorry_hire->advance>0){
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$lorry_hire->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$lorry_hire->advance, 'cr_amount'=>0 ];
            

            // CREDIT INCOME ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$lorry_hire->paid_using, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$lorry_hire->advance ];

           
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }

            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr;


            $this->create_transaction_voucher( $t_data );

        }

    }

}
