<?php

namespace App\Http\Controllers\HR;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Business\Company;
use App\Models\HR\Attendance;
use App\Models\HR\Salary;
use App\Models\HR\SalaryDesc;
use App\Models\Master\Employee;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class SalaryController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Salary');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $salaries = Salary::with([
            'branch:id,branch_name,branch_code',
            'expense:id,account_name',
            'ss_desc'
        ])->where('fin_year',$request->current_year)->get();

        

        return response()->json($salaries);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $salary = Salary::with([
            'branch:id,branch_name,branch_code',
            'expense:id,account_name',
            'ss_desc'
        ])->find($pk);


        return response()->json(['salary' =>$salary]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'Salary');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        $validatedData = $request->validate([
            'salary_number' => 'required|unique:salaries,salary_number',
            'month' => 'required',
            'date' => 'required'
        ], [
            'month.required' => 'Required',
            'salary_number.required' => 'Required',
            'salary_number.unique' => 'Already Exist !',
        ]);

        

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            

            // IN CASE THE CHALLAN TYPE IS 'PRINTED', WE NEED TO CHECK IF THE BOOK IS ACTUALLY ISSUED TO THE BRANCH OR NOT
            $max_number = $this->get_max_number_from_salary_number($request->salary_number);
            
            // SAVING A NEW DS
            $salary = new Salary();
            $salary->branch_id = $request->branch ? $request->branch['id'] : null;
            $salary->max_number = $max_number;
            $salary->salary_number = $request->salary_number;
            $salary->date = $request->date;
            $salary->month = $request->month;
            $salary->exp_id = $request->expense['id'];
            $salary->category = $request->category;
            
            $salary->total_fixed_basic = $request->total_fixed_basic ;
            $salary->total_fixed_spl_pf = $request->total_fixed_spl_pf ;
            $salary->total_fixed_house_rent = $request->total_fixed_house_rent ;
            $salary->total_fixed_conveyance = $request->total_fixed_conveyance ;
            $salary->total_fixed_other = $request->total_fixed_other ;
            $salary->total_fixed_medical = $request->total_fixed_medical ;
            $salary->total_fixed_statutory_bonus = $request->total_fixed_statutory_bonus ;
            $salary->total_fixed_gross_salary = $request->total_fixed_gross_salary ;
            $salary->total_pf_employee = $request->total_pf_employee ;
            $salary->total_esic_employee = $request->total_esic_employee ;
            $salary->total_pt = $request->total_pt ;
            $salary->total_lwf_employee = $request->total_lwf_employee ;
            $salary->total_employee_medical = $request->total_employee_medical ;
            $salary->total_total_employee_deduction = $request->total_total_employee_deduction ;
            $salary->total_net_take_home_salary = $request->total_net_take_home_salary ;
            $salary->total_pf_employer = $request->total_pf_employer ;
            $salary->total_esic_employer = $request->total_esic_employer ;
            $salary->total_lwf_employer = $request->total_lwf_employer ;
            $salary->total_total_employer_contribution = $request->total_total_employer_contribution ;
            $salary->total_total_ctc = $request->total_total_ctc ;

            $salary->fin_year = $request->current_year;

            $salary->created_by = $request->user->id;

            // Save the model to the database
            $salary->save();

            

            // SAVING SETTLEMENT DETAILS
            $ss_desc = $request->ss_desc;
            foreach ($ss_desc as $dsd) {
                $ds_dc = new SalaryDesc();

                $ds_dc->salary_id = $salary->id;
                $ds_dc->salary_number = $salary->salary_number;
                $ds_dc->emp_id = $dsd['emp_id'];
                $ds_dc->emp_name = $dsd['emp_name'];
                $ds_dc->absent = $dsd['absent'] ?? 0;
                $ds_dc->working = $dsd['working'] ?? 0;
                $ds_dc->salary_on = $dsd['salary_on'] ?? 0;
                $ds_dc->salary_paid = $dsd['salary_paid'] ?? 0;
                $ds_dc->fixed_basic = $dsd['fixed_basic'] ?? 0;
                $ds_dc->fixed_spl_pf = $dsd['fixed_spl_pf'] ?? 0;
                $ds_dc->fixed_house_rent = $dsd['fixed_house_rent'] ?? 0;
                $ds_dc->fixed_conveyance = $dsd['fixed_conveyance'] ?? 0;
                $ds_dc->fixed_other = $dsd['fixed_other'] ?? 0;
                $ds_dc->fixed_medical = $dsd['fixed_medical'] ?? 0;
                $ds_dc->fixed_statutory_bonus = $dsd['fixed_statutory_bonus'] ?? 0;
                $ds_dc->fixed_gross_salary = $dsd['fixed_gross_salary'] ?? 0;
                $ds_dc->pf_employee = $dsd['pf_employee'] ?? 0;
                $ds_dc->esic_employee = $dsd['esic_employee'] ?? 0;
                $ds_dc->pt = $dsd['pt'] ?? 0;
                $ds_dc->lwf_employee = $dsd['lwf_employee'] ?? 0;
                $ds_dc->employee_medical = $dsd['employee_medical'] ?? 0;
                $ds_dc->total_employee_deduction = $dsd['total_employee_deduction'] ?? 0;
                $ds_dc->net_take_home_salary = $dsd['net_take_home_salary'] ?? 0;
                $ds_dc->pf_employer = $dsd['pf_employer'] ?? 0;
                $ds_dc->esic_employer = $dsd['esic_employer'] ?? 0;
                $ds_dc->lwf_employer = $dsd['lwf_employer'] ?? 0;
                $ds_dc->total_employer_contribution = $dsd['total_employer_contribution'] ?? 0;
                $ds_dc->total_ctc = $dsd['total_ctc'] ?? 0;
            
                $ds_dc->fin_year = $request->current_year;
                $ds_dc->save();
            }
            // SETTLEMENT DETAILS SAVED


            // RECORD THE TRANSACTION IN ACCOUNTS
           $this->record_transaction_in_accounts($request,$salary);
            // TRANSACTION RECORDED IN ACCOUNTS

             
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($salary->id);
            $this->createEditLog('created', 'salaries', $salary->id, $salary->salary_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
            
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$salary->id, 'salary_number'=>$salary->salary_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'Salary');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $salary = Salary::find($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($salary->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'salary_number' => 'required',
            'month' => 'required',
            'date' => 'required'
        ], [
            'month.required' => 'Required',
            'salary_number.required' => 'Required',
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            // SAVING A NEW DS
            $salary->date = $request->date;
            
            $salary->total_fixed_basic = $request->total_fixed_basic ;
            $salary->total_fixed_spl_pf = $request->total_fixed_spl_pf ;
            $salary->total_fixed_house_rent = $request->total_fixed_house_rent ;
            $salary->total_fixed_conveyance = $request->total_fixed_conveyance ;
            $salary->total_fixed_other = $request->total_fixed_other ;
            $salary->total_fixed_medical = $request->total_fixed_medical ;
            $salary->total_fixed_statutory_bonus = $request->total_fixed_statutory_bonus ;
            $salary->total_fixed_gross_salary = $request->total_fixed_gross_salary ;
            $salary->total_pf_employee = $request->total_pf_employee ;
            $salary->total_esic_employee = $request->total_esic_employee ;
            $salary->total_pt = $request->total_pt ;
            $salary->total_lwf_employee = $request->total_lwf_employee ;
            $salary->total_employee_medical = $request->total_employee_medical ;
            $salary->total_total_employee_deduction = $request->total_total_employee_deduction ;
            $salary->total_net_take_home_salary = $request->total_net_take_home_salary ;
            $salary->total_pf_employer = $request->total_pf_employer ;
            $salary->total_esic_employer = $request->total_esic_employer ;
            $salary->total_lwf_employer = $request->total_lwf_employer ;
            $salary->total_total_employer_contribution = $request->total_total_employer_contribution ;
            $salary->total_total_ctc = $request->total_total_ctc ;
            

            $salary->updated_by = $request->user->id;

            $salary->save();
            // NEW CHALLAN SAVED
            
            // SAVING PACKAGE DETAILS
            $newly_added_ss_desc = $request->newly_added_ss_desc;
            foreach ($newly_added_ss_desc as $nald) {
                $ds_dc = new SalaryDesc();
                $ds_dc->salary_id = $salary->id;
                $ds_dc->salary_number = $salary->salary_number; 
                $ds_dc->emp_id = $nald['emp_id'];
                $ds_dc->emp_name = $nald['emp_name'];
                $ds_dc->absent = $nald['absent'] ?? 0;
                $ds_dc->working = $nald['working'] ?? 0;
                $ds_dc->salary_on = $nald['salary_on'] ?? 0;
                $ds_dc->salary_paid = $nald['salary_paid'] ?? 0;
                $ds_dc->fixed_basic = $nald['fixed_basic'] ?? 0;
                $ds_dc->fixed_spl_pf = $nald['fixed_spl_pf'] ?? 0;
                $ds_dc->fixed_house_rent = $nald['fixed_house_rent'] ?? 0;
                $ds_dc->fixed_conveyance = $nald['fixed_conveyance'] ?? 0;
                $ds_dc->fixed_other = $nald['fixed_other'] ?? 0;
                $ds_dc->fixed_medical = $nald['fixed_medical'] ?? 0;
                $ds_dc->fixed_statutory_bonus = $nald['fixed_statutory_bonus'] ?? 0;
                $ds_dc->fixed_gross_salary = $nald['fixed_gross_salary'] ?? 0;
                $ds_dc->pf_employee = $nald['pf_employee'] ?? 0;
                $ds_dc->esic_employee = $nald['esic_employee'] ?? 0;
                $ds_dc->pt = $nald['pt'] ?? 0;
                $ds_dc->lwf_employee = $nald['lwf_employee'] ?? 0;
                $ds_dc->employee_medical = $nald['employee_medical'] ?? 0;
                $ds_dc->total_employee_deduction = $nald['total_employee_deduction'] ?? 0;
                $ds_dc->net_take_home_salary = $nald['net_take_home_salary'] ?? 0;
                $ds_dc->pf_employer = $nald['pf_employer'] ?? 0;
                $ds_dc->esic_employer = $nald['esic_employer'] ?? 0;
                $ds_dc->lwf_employer = $nald['lwf_employer'] ?? 0;
                $ds_dc->total_employer_contribution = $nald['total_employer_contribution'] ?? 0;
                $ds_dc->total_ctc = $nald['total_ctc'] ?? 0;

                $ds_dc->fin_year = $request->current_year;
                $ds_dc->save();
            }
            // PACKAGE DETAILS SAVED

            // DELETING EXISTING PACKAGE DETAILS
            $deleted_ss_desc = $request->deleted_ss_desc;
            if( !empty($deleted_ss_desc) ){
                foreach ($deleted_ss_desc as $dcd) {
                    $lhd = SalaryDesc::find($dcd);
                    $lhd->delete();
                }
            }
            //DELETED EXISTING PACKAGE DETAILS


            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request,$salary);
            // TRANSACTION RECORDED IN ACCOUNTS
            

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($salary->id);
            $this->createEditLog('updated', 'salaries', $salary->id, $salary->salary_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'Salary');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);
        
        $salary = Salary::find($pk);


        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($salary->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();


            $ss_desc_ids = SalaryDesc::select('id')->where('salary_id', $pk)->where('fin_year',$request->current_year)->pluck('id')->toArray();

            $ss_desc = SalaryDesc::whereIn('id', $ss_desc_ids)->delete();
        

            //FIND AND DELETE THE BRANCH
            $salary->delete();
            // COMMISSIONS WILL BE CASCADED AUTOMATICALLY

            //DELETE THE TRANSACTION VOUCHER
            $t_data = [];
            $t_data['t_against'] = 'Salary';
            $t_data['t_against_id'] = $salary->id;
            $t_data['t_against_no'] = $salary->salary_number;
            $t_data['fin_year'] = $request->current_year;

           $this->delete_transaction_voucher($t_data);


            $this->createEditLog('deleted', 'salaries', $salary->id, $salary->salary_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = Salary::select('id', 'salary_number')->where('fin_year',$request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }
        
        }

        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('salary_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $salaries = $query->get();
        return response(['salaries' => $salaries, 'draw' => $request->draw]);
    }


    public function fetch_desc_for_salary(Request $request){
        $branch = $request->branch;
        $month = $request->month;
        $category = $request->category;
        
        $data = [];
    

        $query = Employee::select('id','emp_name','casual_leaves','earning_leaves','sick_leaves','fixed_basic','fixed_spl_pf','fixed_house_rent','fixed_conveyance','fixed_other','fixed_medical','fixed_statutory_bonus','fixed_gross_salary','pf_employee_percent','pf_employee','esic_employee_percent','esic_employee','pt','lwf_employee','employee_medical','total_employee_deduction','net_take_home_salary','pf_employer_percent','pf_employer','esic_employer_percent','esic_employer','lwf_employer','total_employer_contribution','total_ctc')
                ->where('category', $category);

    
        if($branch){
           $query = $query->where('branch_id', $branch['id']);
        }

        $currentYear = $request->current_year;

        $employees = $query->whereDoesntHave('salary_desc', function ($query) use ($month,$currentYear) {
            $query->whereHas('salary', function ($query) use ($month,$currentYear) {
                $query->where('month', $month)->where('fin_year',$currentYear);
            });
        })->get();

        // Map data in the format required by the frontend
        foreach ($employees as $employee) {

            // GET ATTENDANCE WHERE ATTEND DATE (YYYY-MM-DD) IS IN THE MONTH (1-12)
            $attendance = Attendance::where('emp_id', $employee->id)->whereMonth('attend_date', $month)->where('fin_year', $request->current_year);

            $year = $this->get_start_end_year($request->current_year);
            $financialYear = ($month >= 4) ? $year->start_year : $year->end_year;
            $daysInMonth = Carbon::createFromDate($financialYear, $month, 1)->daysInMonth;

            $absent = $attendance->where('attendance','A')->count();
            $half_day = $attendance->where('attendance','HD')->count() / 2;
            
            $working = $daysInMonth - $absent - $half_day;

            $per_day = $employee->fixed_basic / $daysInMonth;

            $salary_on =   number_format($working * $per_day, 2, '.', '');


            // Find basic pay here from $employee->fixed_basic

            $data[] = (object) [
                'attendance' => $attendance,
                'id' => '',
                'salary_id' => '',
                'emp_id' => $employee->id,
                'emp_name' => $employee->emp_name,
                'absent' => $absent,
                'working' => $working,
                'salary_on' => $employee->fixed_basic,
                'salary_paid' => $salary_on,
                'fixed_basic' => $salary_on,
                'fixed_spl_pf' => $employee->fixed_spl_pf,
                'fixed_house_rent' => $employee->fixed_house_rent,
                'fixed_conveyance' => $employee->fixed_conveyance,
                'fixed_other' => $employee->fixed_other,
                'fixed_medical' => $employee->fixed_medical,
                'fixed_statutory_bonus' => $employee->fixed_statutory_bonus,
                'fixed_gross_salary' => $employee->fixed_gross_salary,
                'pf_employee_percent' => $employee->pf_employee_percent,
                'pf_employee' => $employee->pf_employee,
                'esic_employee_percent' => $employee->esic_employee_percent,
                'esic_employee' => $employee->esic_employee,
                'pt' => $employee->pt,
                'lwf_employee' => $employee->lwf_employee,
                'employee_medical' => $employee->employee_medical,
                'total_employee_deduction' => $employee->total_employee_deduction,
                'net_take_home_salary' => $employee->net_take_home_salary,
                'pf_employer_percent' => $employee->pf_employer_percent,
                'pf_employer' => $employee->pf_employer,
                'esic_employer_percent' => $employee->esic_employer_percent,
                'esic_employer' => $employee->esic_employer,
                'lwf_employer' => $employee->lwf_employer,
                'total_employer_contribution' => $employee->total_employer_contribution,
                'total_ctc' => $employee->total_ctc,
                'checked' => true,
            ];
        }


        

        return response()->json(['against_details' => $data]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Salary');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Salary::with([
                'branch:id,branch_name,branch_code',
                'expense:id,account_name',
                'ss_desc'
                ])->where('fin_year', $request->current_year)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('salary_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = Salary::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $salariesQuery = Salary::select('id', 'salary_number')->where('fin_year', $request->current_year);
        
        
        
        $salaries = $salariesQuery->orderBy('id', 'desc')->take(10)->get();
        
        return response()->json(['salaries' => $salaries]);
    }
    


    public function get_next_salary_number(Request $request){
        $type='SYSTEM';

        if($type=='SYSTEM'){
            $company_code = Company::select('company_code')->find($request->company['id'])['company_code'];

            $max = Salary::max('max_number');
            
            $salary_number = $company_code.'-'.sprintf('%05d', $max+1);
            
            return response()->json(['salary_number' => strtoupper($salary_number), 'max_number' => $max+1]);
        }
    }

    protected function get_data_for_obj($pk){
        // FETCH AND SEND
        $salary = Salary::with([
            'branch:id,branch_name,branch_code',
            'expense:id,account_name',
            'ss_desc'
        ])->find($pk);

    
        return ['salary' =>$salary];
    }


    public function get_custom_fields(Request $request){
        $salary = Salary::select($request->custom_fields)->with($request->with_relations)->find($request->lh_id);
        return response()->json(['salary' => $salary]);
    }

    protected function get_max_number_from_salary_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected  function  record_transaction_in_accounts($request,$salary){
        return;
        $t_data = [];
        $t_data['t_against'] = 'Salary';
        $t_data['t_against_id'] = $salary->id;
        $t_data['t_against_no'] = $salary->salary_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);

        $t_data['branch'] = $request->branch;
        $t_data['voucher_date'] = $request->date;
        $t_data['voucher_time'] = null;
        
        $t_data['t_against_narration'] = "";
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];

        $total_dr = $salary->total_net_take_home_salary;
        $total_cr = 0;

        $t_data['voucher_accounting_details'][] = ['account_id'=>$salary->exp_id, 'nature'=>'dr' , 'dr_amount'=>$total_dr, 'cr_amount'=>0 ];
          
        $t_data['cancellable'] = 1;
        if($total_dr>0 && $salary->exp_id){

            $salary_desc = SalaryDesc::with('employee:id,emp_id,emp_name,account_id')->where('salary_id', $salary->id)->get();

            foreach ($salary_desc as $sd) {
                $net_take_home_salary = $sd['net_take_home_salary'];
                $emp_acc_id = $sd['employee']['account_id'];
                $t_data['voucher_accounting_details'][] = ['account_id'=>$emp_acc_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$net_take_home_salary ];
            } 
            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }


            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }



            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr;


            $this->create_transaction_voucher( $t_data );
        }

    }

}
