<?php

namespace App\Http\Controllers\Leads;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Business\Company;
use App\Models\Leads\FollowUp;
use App\Models\Leads\Lead;
use App\Models\Leads\Quotation;
use App\Models\Leads\QuotationDesc;
use Illuminate\Support\Facades\DB;

use Carbon\Carbon;

class QuotationController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;

    public function index(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'Quotation');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        

        // FETCH AND SEND ALL
        $query = Quotation::with(['lead:id,lead_number', 'quotation_desc:id,quotation_id,lead_id,item_id,qty', 'quotation_desc.item:id,item_name'])->where('fin_year', $request->current_year);
        
        if ($request->user->power < 1) {
            $query->where(function($subquery) use ($request) {
                $subquery->whereHas('lead', function ($q) use ($request) {
                    $q->where('assigned_to_id', $request->user->id)->orWhere('surveyor_id', $request->user->id);
                });
            });
        }

        $quotations = $query->all();

        return response()->json($quotations);
    }

    public function show(Request $request, $pk)
    {
        // FETCH AND SEND
        $quotation = Quotation::with(['lead:id,lead_number','quotation_desc:id,quotation_id,lead_id,item_id,qty','quotation_desc.item:id,item_name,category_id','quotation_desc.item.category:id,category_name','follow_ups','follow_ups.follow_up_by:id,username'])->find($pk);

        // Map quotation_desc like { id: 1, item_id: 1, item_name: 'item1', qty: 1 }
        $quotation_desc = $quotation->quotation_desc->map(function ($item) {
            return [
                'quotation_desc_id' => $item->id,
                'lead_id' => $item->lead_id,
                'quotation_id' => $item->quotation_id,
                'category_name' => $item->item->category->category_name,
                'id' => $item->item_id,
                'item_name' => $item->item->item_name,
                'qty' => $item->qty,
            ];
        });

        return response()->json(['quotation' => $quotation, 'quotation_desc' => $quotation_desc]);
    }

    public function store(Request $request)
    {

        $test = $this->is_new_permitted($request->user->id, 'Quotation');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $validatedData = $request->validate([
            'quotation_number' => 'required|unique:quotations,quotation_number',
            'quotation_number' => 'required'
        ], [
            'quotation_number.required' => 'Required',
            'quotation_number.unique' => 'Already Exist!',
            'quotation_number.required' => 'Required',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            
            $max_number = $this->get_max_number_from_quotation_number($request->quotation_number);

            // SAVING A NEW Lead
            $quotation = new Quotation();
            $quotation->quotation_number = $request->quotation_number;
            $quotation->lead_id = $request->lead['id'];

            // Get the current date
            $quotation->quotation_date = Carbon::today();
            // Get the current time
            $quotation->quotation_time = Carbon::now()->format('H:i:s');

            $quotation->max_number = $max_number;
            $quotation->name = $request->name;
            $quotation->email = $request->email;
            $quotation->mob_number = $request->mob_number;
            $quotation->mob_number2 = $request->mob_number2;
            $quotation->source = $request->source;
            $quotation->destination = $request->destination;
            $quotation->vehicle_type = $request->vehicle_type;
            $quotation->pickup_date = $request->pickup_date;
            $quotation->drop_date = $request->drop_date;
            $quotation->pickup_address1 = $request->pickup_address1;
            $quotation->pickup_address2 = $request->pickup_address2;
            $quotation->pickup_state = $request->pickup_state;
            $quotation->pickup_landmark = $request->pickup_landmark;
            $quotation->pickup_packing_service = $request->pickup_packing_service;
            $quotation->pickup_loading_service = $request->pickup_loading_service;
            $quotation->pickup_parking = $request->pickup_parking;
            $quotation->pickup_floor = $request->pickup_floor;
            $quotation->pickup_lift = $request->pickup_lift;
            $quotation->drop_address1 = $request->drop_address1;
            $quotation->drop_address2 = $request->drop_address2;
            $quotation->drop_state = $request->drop_state;
            $quotation->drop_landmark = $request->drop_landmark;
            $quotation->drop_unpacking_service = $request->drop_unpacking_service;
            $quotation->drop_unloading_service = $request->drop_unloading_service;
            $quotation->drop_parking = $request->drop_parking;
            $quotation->drop_floor = $request->drop_floor;
            $quotation->drop_lift = $request->drop_lift;
            

            $quotation->freight_chg = $request->freight_chg;
            $quotation->freight_chg_vehicle = $request->freight_chg_vehicle;
            $quotation->vehicle_model = $request->vehicle_model;
            $quotation->statistical_chg = $request->statistical_chg;
            $quotation->sub_total1 = $request->sub_total1;
            $quotation->packing_chg = $request->packing_chg;
            $quotation->loading_chg = $request->loading_chg;
            $quotation->unloading_chg = $request->unloading_chg;
            $quotation->unpacking_chg = $request->unpacking_chg;
            $quotation->pickup_carpenter = $request->pickup_carpenter;
            $quotation->pickup_ac_man = $request->pickup_ac_man;
            $quotation->pickup_electrician = $request->pickup_electrician;
            $quotation->drop_carpenter = $request->drop_carpenter;
            $quotation->drop_ac_man = $request->drop_ac_man;
            $quotation->drop_electrician = $request->drop_electrician;
            $quotation->handyman_chg = $request->handyman_chg;
            $quotation->risk_chg_applied = $request->risk_chg_applied;
            $quotation->risk_chg_on = $request->risk_chg_on;
            $quotation->risk_chg_rate = $request->risk_chg_rate;
            $quotation->risk_chg = $request->risk_chg;
            $quotation->warehousing_chg = $request->warehousing_chg;
            $quotation->sub_total2 = $request->sub_total2;
            $quotation->gst_rate = $request->gst_rate;
            $quotation->gst_chg = $request->gst_chg;
            $quotation->toll_tax = $request->toll_tax;
            $quotation->grand_total = $request->grand_total;
            $quotation->token_advance = $request->token_advance;
            $quotation->balance_amount = $request->balance_amount;


            $quotation->quotation_status = 'Yet To Be Contacted';

            $quotation->fin_year = $request->current_year;

            $quotation->created_by = $request->user->id;
            $quotation->save();
            // NEW Lead SAVED
            
            $quotation_descs = $request->quotation_desc;

            if( !empty($quotation_descs) ){
            foreach ($quotation_descs as $q_desc) {
                if (!$q_desc['item_name'] || $q_desc['qty']<1) {
                    continue;
                }
                
                $q_des = new QuotationDesc();
                $q_des->quotation_id = $quotation->id;
                $q_des->lead_id = $quotation->lead_id;
                $q_des->item_id = $q_desc['id'];
                $q_des->qty = $q_desc['qty'];

                $q_des->fin_year = $request->current_year;
                $q_des->save();
            }
            }

            $newly_added_desc = $request->newly_added_quotation_desc;
            if( !empty($newly_added_desc) ) {
                foreach ($newly_added_desc as $nad) {
                    if (!$nad['item'] || $nad['qty']<1) {
                        continue;
                    }
                    
                    $q_des = new QuotationDesc();
                    $q_des->quotation_id = $quotation->id;
                    $q_des->lead_id = $quotation->lead_id;
                    $q_des->item_id = $nad['item']['id'];
                    $q_des->qty = $nad['qty'];
                    $q_des->save();
                }
            }
            

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($quotation->id);
            $this->createEditLog('created', 'quotations', $quotation->id, $quotation->quotation_number, null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG


            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        }
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk)
    {
        $test = $this->is_edit_permitted($request->user->id, 'Quotation');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if (!$test_edit) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $quotation = Quotation::find($pk);

        if($request->user->power < 1){
            $lea = Lead::select('id','assinged_to_id','surveyor_id')->where('id',$quotation->lead_id)->first();
            if($lea->assinged_to_id != $request->user->id && $lea->surveyor_id != $request->user->id){
                return response()->json(['Permission Denied For The Action'], 401);
            }
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        if ($edit_valid_till_hours != null) {
            $validity = Carbon::parse($quotation->created_at)->addHours($edit_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }


        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // UPDATING THE Lead
            $quotation->name = $request->name;
            $quotation->email = $request->email;
            $quotation->mob_number = $request->mob_number;
            $quotation->mob_number2 = $request->mob_number2;
            $quotation->source = $request->source;
            $quotation->destination = $request->destination;
            $quotation->vehicle_type = $request->vehicle_type;
            $quotation->pickup_date = $request->pickup_date;
            $quotation->drop_date = $request->drop_date;
            $quotation->pickup_address1 = $request->pickup_address1;
            $quotation->pickup_address2 = $request->pickup_address2;
            $quotation->pickup_state = $request->pickup_state;
            $quotation->pickup_landmark = $request->pickup_landmark;
            $quotation->pickup_packing_service = $request->pickup_packing_service;
            $quotation->pickup_loading_service = $request->pickup_loading_service;
            $quotation->pickup_parking = $request->pickup_parking;
            $quotation->pickup_floor = $request->pickup_floor;
            $quotation->pickup_lift = $request->pickup_lift;
            $quotation->drop_address1 = $request->drop_address1;
            $quotation->drop_address2 = $request->drop_address2;
            $quotation->drop_state = $request->drop_state;
            $quotation->drop_landmark = $request->drop_landmark;
            $quotation->drop_unpacking_service = $request->drop_unpacking_service;
            $quotation->drop_unloading_service = $request->drop_unloading_service;
            $quotation->drop_parking = $request->drop_parking;
            $quotation->drop_floor = $request->drop_floor;
            $quotation->drop_lift = $request->drop_lift;


            $quotation->freight_chg = $request->freight_chg;
            $quotation->freight_chg_vehicle = $request->freight_chg_vehicle;
            $quotation->vehicle_model = $request->vehicle_model;
            $quotation->statistical_chg = $request->statistical_chg;
            $quotation->sub_total1 = $request->sub_total1;
            $quotation->packing_chg = $request->packing_chg;
            $quotation->loading_chg = $request->loading_chg;
            $quotation->unloading_chg = $request->unloading_chg;
            $quotation->unpacking_chg = $request->unpacking_chg;
            $quotation->pickup_carpenter = $request->pickup_carpenter;
            $quotation->pickup_ac_man = $request->pickup_ac_man;
            $quotation->pickup_electrician = $request->pickup_electrician;
            $quotation->drop_carpenter = $request->drop_carpenter;
            $quotation->drop_ac_man = $request->drop_ac_man;
            $quotation->drop_electrician = $request->drop_electrician;
            $quotation->handyman_chg = $request->handyman_chg;
            $quotation->risk_chg_applied = $request->risk_chg_applied;
            $quotation->risk_chg_on = $request->risk_chg_on;
            $quotation->risk_chg_rate = $request->risk_chg_rate;
            $quotation->risk_chg = $request->risk_chg;
            $quotation->warehousing_chg = $request->warehousing_chg;
            $quotation->sub_total2 = $request->sub_total2;
            $quotation->gst_rate = $request->gst_rate;
            $quotation->gst_chg = $request->gst_chg;
            $quotation->toll_tax = $request->toll_tax;
            $quotation->grand_total = $request->grand_total;
            $quotation->token_advance = $request->token_advance;
            $quotation->balance_amount = $request->balance_amount;

            $quotation->updated_by = $request->user->id;
            $quotation->save();
            // Lead UPDATED


            $quotation_desc_delete = QuotationDesc::where('quotation_id',$pk)->delete();


            $quotation_descs = $request->quotation_desc;

            if( !empty($quotation_descs) ){
                foreach ($quotation_descs as $q_desc) {
                    if (!$q_desc['item_name'] || $q_desc['qty']<1) {
                        continue;
                    }
                    
                    $q_des = new QuotationDesc();
                    $q_des->quotation_id = $quotation->id;
                    $q_des->lead_id = $quotation->lead_id;
                    $q_des->item_id = $q_desc['id'];
                    $q_des->qty = $q_desc['qty'];
                    $q_des->fin_year = $request->current_year;
                    $q_des->save();
                }
            }
            

            $newly_added_desc = $request->newly_added_quotation_desc;
            if( !empty($newly_added_desc) ) {
                foreach ($newly_added_desc as $nad) {
                    if (!$nad['item'] || $nad['qty']<1) {
                        continue;
                    }
                    
                    $q_des = new QuotationDesc();
                    $q_des->quotation_id = $quotation->id;
                    $q_des->lead_id = $quotation->lead_id;
                    $q_des->item_id = $nad['item']['id'];
                    $q_des->qty = $nad['qty'];
                    $q_des->fin_year = $request->current_year;
                    $q_des->save();
                }
            }
            


            // SAVING NEWLY ADDED FOLLOW UPS
            $newly_added_follow_ups = $request->newly_added_follow_ups;
            foreach ($newly_added_follow_ups as $nafu) {
                if(!$nafu['status'])
                continue;
                
                $fu = new FollowUp();
                $fu->quotation_id = $quotation->id;
                $fu->status = $nafu['status'];
                $fu->date = $nafu['date'];
                $fu->time = $nafu['time'];
                $fu->remarks = $nafu['remarks'];
                $fu->follow_up_by_id = $request->user->id;
                $fu->against = 'Quotation';
                $fu->fin_year = $request->current_year;
                $fu->save();
            }
            // NEWLY ADDED FOLLOW UPS SAVED


            // SAVING UPDATES ON  FollowUPS
            $updated_follow_up_details = $request->updated_follow_up_details;
            if( !empty($updated_follow_up_details) ){
                foreach ($updated_follow_up_details as $ufud) {
                    if(!$ufud['status'])
                       continue;

                    $fu = FollowUp::find($ufud['id']);
                    $fu->status = $ufud['status'];
                    $fu->date = $ufud['date'];
                    $fu->time = $ufud['time'];
                    $fu->remarks = $ufud['remarks'];
                    $fu->save();

                }

            }
            // UPDATES SAVED ON FollowUPS

            // DELETING EXISTING Follow UPS
            $deleted_follow_up_details = $request->deleted_follow_up_details;
            if( !empty($deleted_follow_up_details) ){
                    $fu = FollowUp::whereIn('id', $deleted_follow_up_details);
                    $fu->delete();
            }
            //DELETED EXISTING Follow UPS

            $latest_status = FollowUp::select('status')->where('quotation_id',$pk)->orderBy('id','desc')->first();
            if($latest_status){
                $quotation->quotation_status = $latest_status->status;
                $quotation->save();
            }
            

            $newVal = $this->get_data_for_obj($quotation->id);
            $this->createEditLog('updated', 'quotations', $quotation->id, $quotation->quotation_number, $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG


            // TRANSACTIONS COMMITTED
            DB::commit();

            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk)
    {
        $test = $this->is_delete_permitted($request->user->id, 'Quotation');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if (!$test_delete) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $quotation = Quotation::find($pk);

        if($request->user->power < 1){
            $lea = Lead::select('id','assinged_to_id','surveyor_id')->where('id',$quotation->lead_id)->first();
            if($lea->assinged_to_id != $request->user->id && $lea->surveyor_id != $request->user->id){
                return response()->json(['Permission Denied For The Action'], 401);
            }
        }

        if ($delete_valid_till_hours != null) {
            $validity = Carbon::parse($quotation->created_at)->addHours($delete_valid_till_hours);
            if (!$validity->isFuture()) {
                return response()->json(['Permission Timed Out'], 500);
            }
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
           

        $quotation_desc_delete = QuotationDesc::where('quotation_id',$pk)->delete();
        $follow_up_delete = FollowUp::where('quotation_id',$pk)->delete();

        // FIND AND DELETE THE Lead
        $quotation->delete();

        $this->createEditLog('deleted', 'quotations', $quotation->id, $quotation->quotation_number, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request)
    {
        // WHILE SEARCHING THIS WILL RUN
        $query = Quotation::select('id', 'quotation_number')->where('fin_year', $request->current_year);

        if($request->user->power < 1){
            $query->where(function($subquery) use ($request) {
                $subquery->whereHas('lead', function ($q) use ($request) {
                    $q->where('assigned_to_id', $request->user->id)->orWhere('surveyor_id', $request->user->id);
                });
            });
        }



        // Apply any additional filters based on other columns as needed
        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

           // Loop through the whereFilters array
           foreach ($whereFilters as $whereFilter) {
               // Apply the filter to the query
               $query->where($whereFilter['key'], $whereFilter['value']);
           }

           // Loop through the orWhereFilters array
           foreach ($orWhereFilters as $orWhereFilter) {
               // Apply the filter to the query
               $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
           }
        
        }
        
        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('quotation_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('quotation_number', 'asc');
        // Retrieve the filtered 
        $quotations = $query->get();
        return response(['quotations' => $quotations, 'draw' => $request->draw]);
    }

    public function autocomplete_quotation_for_gr(Request $request) {
        $quotations = Quotation::select('id', 'quotation_number')
            ->whereDoesntHave('book_gr')
            ->get();
    
        return response()->json(['quotations' => $quotations]);
    }

    public function get_quotation_info_for_gr(Request $request, $pk){
        $quotation = Quotation::find($pk);
        
        return response()->json(['quotation' => $quotation]);
    }

    public function datatable(Request $request)
    {
        $test = $this->is_read_permitted($request->user->id, 'Quotation');
        if (!$test) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;
        $status = $request->quotation_status;
        $type = $request->type;

        $query = Quotation::select('id','lead_id','quotation_number','source','destination','name','mob_number','pickup_address1','drop_address1','quotation_date','quotation_time','quotation_status', 'pickup_state','drop_state')
               ->with(['lead:id,lead_number','quotation_desc:id,quotation_id,lead_id,item_id,qty','quotation_desc.item:id,item_name'])->where('fin_year', $request->current_year)->orderBy($column, $dir);

        if($request->user->power < 1){
            $query->where(function($subquery) use ($request) {
                $subquery->whereHas('lead', function ($q) use ($request) {
                    $q->where('assigned_to_id', $request->user->id)->orWhere('surveyor_id', $request->user->id);
                });
            });
        }

        if($status){
            $query->where(function ($query) use ($status) {
                $query->where('quotation_status', $status);
            });
        }
        
        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('quotation_number', 'like', $searchValue . '%');
            });
        }



        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }


    public function get_next_quotation_number(Request $request){
        $type='SYSTEM';

        if($type=='SYSTEM'){
            $company_code = Company::select('company_code')->find($request->company['id'])['company_code'];

            $max = Quotation::max('max_number');
            //$quotation_number = sprintf('%06d', $max+1);
            $quotation_number = $company_code.'-'.sprintf('%06d',$max+1);

            
            return response()->json(['quotation_number' => strtoupper($quotation_number), 'max_number' => $max+1]);
        }
    }


    public function check_unique(Request $request)
    {
        $key = $request->key;
        $value = $request->value;

        if (!$value) {
            return response()->json(['exists' => false]);
        }

        $exists = Quotation::where($key, $value)->exists();
        if ($exists) {
            return response()->json(['errors' => [$key  => ['Already Exists !']]], 422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_print_data(Request $request, $pk)
    {
        // FETCH AND SEND
        $quotation = Quotation::with(['lead:id,lead_number','quotation_desc:id,quotation_id,lead_id,item_id,qty','quotation_desc.item:id,item_name,category_id','quotation_desc.item.category:id,category_name','follow_ups','follow_ups.follow_up_by:id,username'])->find($pk);

        // Map quotation_desc like { id: 1, item_id: 1, item_name: 'item1', qty: 1 }
        $quotation_desc = $quotation->quotation_desc->map(function ($item) {
            return [
                'quotation_desc_id' => $item->id,
                'lead_id' => $item->lead_id,
                'quotation_id' => $item->quotation_id,
                'category_name' => $item->item->category->category_name,
                'id' => $item->item_id,
                'item_name' => $item->item->item_name,
                'qty' => $item->qty,
            ];
        });

        return response()->json(['quotation' => $quotation, 'quotation_desc' => $quotation_desc]);
    }

    protected function get_data_for_obj($pk)
    {
        $quotation = Quotation::with(['lead:id,lead_number','quotation_desc:id,quotation_id,lead_id,item_id,qty','quotation_desc.item:id,item_name','follow_ups','follow_ups.follow_up_by:id,username'])->find($pk);
        return ['quotation' => $quotation];
    }

    protected function get_max_number_from_quotation_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
        
    }
}
