<?php

namespace App\Http\Controllers\Master;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Master\LateDeduction;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class LateDeductionController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        
        // Fetch all the Late Deductions with the relationships on acc_party, source, destination, consignor, consignee
        $late_deduction = LateDeduction::when($request->vehicle_type, function ($query) use ($request) {
            $query->where('vehicle_type', $request->vehicle_type);
        })
        ->get();
        

        return response()->json(['late_deduction' => $late_deduction]);

    }

    public function show(Request $request, $pk){
            $late_deduction = LateDeduction::when($request->vehicle_type, function ($query) use ($request) {
                $query->where('vehicle_type', $request->vehicle_type);
            })->find($pk);

            
            return response()->json(['late_deduction' => $late_deduction]);
    }

    public function store(Request $request){
        

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            // Loop through $request->late_deductions and create new LateDeduction for each of them.
            foreach ($request->late_deduction_list as $index => $pl) {
                if( !$pl['vehicle_type'] && !$pl['from_hrs'] && !$pl['rate'])
                continue;
                 
                // Check if a similar Late Deduction already exists
                $exists = $this->find_duplicate($pl);
                $ind = $index + 1;
                if ($exists) {
                    return response()->json(['errors' => ['type'=>'Duplicate Entry', 'msg'=>"Mathing Record At Price Number ".$exists['id'], 'index'=>$ind, 'id'=>$exists['id']]], 422);
                }

                $pri_lis = new LateDeduction();
                $pri_lis->vehicle_type = $pl['vehicle_type'];
                $pri_lis->from_hrs = $pl['from_hrs'];
                $pri_lis->to_hrs = $pl['to_hrs'];
                $pri_lis->rate = $pl['rate'];
                
                $pri_lis->created_by = $request->user->id;
                $pri_lis->save();

                $newVal = $this->get_data_for_obj($pri_lis->id);
                $this->createEditLog('created', 'late_deductions', $pri_lis->id, $pl['vehicle_type'] , null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }
            
            DB::commit();
            
            return response()->json(['success' => 'Late Deduction Created Successfully']);

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['errors' => ['late_deduction' => [$e->getMessage()]]], 422);
        }    



    }

    public function update(Request $request){

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            
            // Loop through $request->late_deductions and create new LateDeduction for each of them.
            foreach ($request->newly_added_late_deduction_list as $index => $napl) {
                if(!$napl['vehicle_type'] && !$napl['from_hrs'] && !$napl['rate']){
                    continue;
                }

                // Check if a similar Late Deduction already exists
                $exists = $this->find_duplicate($napl);
                $ind = $index + 1;
                if ($exists) {
                    return response()->json(['errors' => ['type'=>'Duplicate Entry', 'msg'=>"Mathing Record At Price Number ".$exists['id'], 'index'=>$ind, 'id'=>$exists['id']]], 422);
                }

                $pri_lis = new LateDeduction();
                $pri_lis->vehicle_type = $napl['vehicle_type'];
                $pri_lis->from_hrs = $napl['from_hrs'];
                $pri_lis->to_hrs = $napl['to_hrs'];
                $pri_lis->rate = $napl['rate'];
                $pri_lis->created_by = $request->user->id;
                $pri_lis->save();


                $newVal = $this->get_data_for_obj($pri_lis->id);
                $this->createEditLog('created', 'late_deductions', $pri_lis->id, $napl['vehicle_type'] , null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }
             
            // Loop through $request->updated_late_deduction and update LateDeduction for each of them.
            foreach ($request->updated_late_deduction_list as $index => $upl) {
                if(!$upl['vehicle_type'] && !$upl['from_hrs'] && !$upl['rate']){
                    continue;
                }

                // Check if a similar Late Deduction already exists
                $exists = $this->find_duplicate($upl);
                $ind = $index + 1;
                if ($exists) {
                    if($exists['id'] != $upl['id'])
                    return response()->json(['errors' => ['type'=>'Duplicate Entry', 'msg'=>"Mathing Record At Price Number ".$exists['id'], 'index'=>$ind, 'id'=>$exists['id']]], 422);
                }

                // FETCH OLD VALUE FOR LOG
                $oldValue = $this->get_data_for_obj($upl['id']);

                $pri_lis = LateDeduction::find($upl['id']);
                $pri_lis->vehicle_type = $upl['vehicle_type'];
                $pri_lis->from_hrs = $upl['from_hrs'];
                $pri_lis->to_hrs = $upl['to_hrs'];
                $pri_lis->rate = $upl['rate'];

                $pri_lis->updated_by = $request->user->id;
                $pri_lis->save();

                // Save Edit Log
                $newVal = $this->get_data_for_obj($pri_lis->id);
                $this->createEditLog('updated', 'late_deductions', $pri_lis->id, $upl['vehicle_type'] , $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }

            // Loop through $request->deleted_late_deduction and delete LateDeduction for each of them.
            foreach ($request->deleted_late_deduction_list as $dpl) {
                // FETCH OLD VALUE FOR LOG
                $oldValue = $this->get_data_for_obj($dpl);

                $pri_lis = LateDeduction::find($dpl);
                $pri_lis->delete();
                
                // Save Edit Log
                $this->createEditLog('deleted', 'late_deductions', $pri_lis->id, $oldValue['late_deduction']['vehicle_type'] , $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }

            DB::commit();

            return response()->json(['success' => 'Late Deduction Updated Successfully']);
        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['errors' => ['late_deduction' => [$e->getMessage()]]], 422);
        }
    
    }

    public function destroy(Request $request){
        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            
            // Loop through $request->deleted_late_deduction and delete LateDeduction for each of them.
            foreach ($request->deleted_late_deduction_list as $dpl) {

                // FETCH OLD VALUE FOR LOG
                $oldValue = $this->get_data_for_obj($dpl);

                

                $pri_lis = LateDeduction::find($dpl);
                $pri_lis->delete();

                // Save Edit Log
                $this->createEditLog('deleted', 'late_deductions', $pri_lis->id, $oldValue['late_deduction']['vehicle_type'] , $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG
            }

            DB::commit();
            return response()->json(['success' => 'Late Deduction Deleted Successfully']);
            
        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['errors' => ['late_deduction' => [$e->getMessage()]]], 422);
        }
    }

    public function check_unique(Request $request){
        // Check if a similar Late Deduction already exists
        $exists = $this->find_duplicate($request->data);
        if ($exists) {
            return response()->json(['errors' => ['type'=>'Duplicate Entry', 'msg'=>"Mathing Record At Price Number ".$exists['id'], 'id'=>$exists['id']]], 422);
        }
        return response("Checked");
    }

    public function fetch_rate_for_ls(Request $request){
        // Start building the query with the selected columns
        $query = LateDeduction::select('id','vehicle_type','from_hrs','to_hrs','rate');
        
        
        $vehicle_type = $request->vehicle['vehicle_type'];
        $late_deduction_qty = $request->late_deduction_qty;


        $query->where(function ($query) use ($vehicle_type) {
            $query->where('vehicle_type', $vehicle_type)
                ->orWhereNull('vehicle_type');
        });


        if ($query->count() > 1) {
            if ($late_deduction_qty) {
                $query->where('to_hrs', '>', $late_deduction_qty)
                    ->orWhereNull('to_hrs');
            }
            $query->orderBy('to_hrs', 'asc');
        }
        

        // Execute the query and get the results
        $pri_lis = $query->first();

        return response()->json(['pri_lis'=>$pri_lis]);
    }


    protected function get_data_for_obj($pk){
        $late_deduction = LateDeduction::find($pk);

            
        return ['late_deduction' => $late_deduction];
    }

    protected function find_duplicate($item){
        $from_hrs = $item['from_hrs'];
        $to_hrs = $item['to_hrs'];
        $vehicle_type = $item['vehicle_type'];

        
        $exists = LateDeduction::where('vehicle_type', $vehicle_type)
                    ->where(function ($query) use ($from_hrs, $to_hrs) {
                        $query->whereBetween('from_hrs', [$from_hrs , $to_hrs-1])
                            ->orWhereBetween('to_hrs', [$from_hrs+1, $to_hrs])
                            ->orWhere(function ($query) use ($from_hrs, $to_hrs) {
                                $query->where('from_hrs', '>', $from_hrs)
                                        ->where('to_hrs', '<=', $to_hrs);
                            });
                    })
                    ->when($item['id'], function ($query) use ($item) {
                        $query->where('id', '!=', $item['id']);
                    })
                    ->first();    
                    
        return $exists;
    }
}
