<?php

namespace App\Http\Controllers\Prints;

use App\Http\Controllers\Controller;
use App\Models\Bill\Bill;
use App\Models\Booking\BookGr;
use Illuminate\Support\Facades\DB;
use App\Models\Business\Company;
use Illuminate\Http\Request;

class PrintController extends Controller
{
    protected $company;

    public function open_gr_print(Request $request){
            $this->initialize($request->cc);

            $gr = BookGr::with([
                'branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name', 'consignor:id,party_name', 'consignee:id,party_name', 'vehicle:id,vehicle_number','driver:id,emp_name',
                'gr_desc',
                'gr_desc.material' => function ($query) {
                    $query->select('id', 'material_name');
                },
                'eway_details',
                'acc_party' => function ($query) {
                    $query->select('id', 'account_name','account_type');
                },
                'acc_party.party:id,account_id',
                'acc_party_branch:id,branch_name',
                
                'consignor_branch:id,branch_name',
                'consignee_branch:id,branch_name',
            ])->find($request->pk);
            

            return view('docs.gr_print')->with(['gr' => $gr,'company'=>$this->company]);
    }

    public function open_bill_print(Request $request){
        $this->initialize($request->cc);
        $bill = Bill::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'acc_party_branch:id,branch_name',
            'acc_party.party:id,account_id,party_name,address1,address2,state,tele_number,gst_number',
            'income:id,account_name',

            'bill_desc',
            'bill_desc.bookGr:id,gr_number,gr_date,consignor_id,consignee_id,source_id,destination_id,consignor_name,consignee_name,source_name,destination_name',
            'bill_desc.bookGr.consignor:id,party_name',
            'bill_desc.bookGr.consignee:id,party_name',
            'bill_desc.bookGr.source:id,station_name',
            'bill_desc.bookGr.destination:id,station_name',
            'bill_desc.pod:id,pod_number',
            
            'bill_desc.bookGr.vehicle:id,vehicle_number,vehicle_type',

            'bill_desc.loading_slip'
        ])->find($request->pk);


        $bill->bill_desc->transform(function ($desc) {
            $data = [
                'bill_desc_id' => $desc->id,
                'bill_id' => $desc->bill_id,
                'total_without_gst' => $desc->total_without_gst,
                'igst_amount' => $desc->igst_amount,
                'cgst_amount' => $desc->cgst_amount,
                'sgst_amount' => $desc->sgst_amount,
                'rounded_off' => $desc->rounded_off,
                'grand_total' => $desc->grand_total,
            ];

            
        
            if ($desc->selection_type == 'GR') {
                $data += [
                    'gr_id' => $desc->gr_id,
                    'gr_number' => $desc->gr_number,
                    'gr_date' => $desc->bookGr->gr_date,
                    'consignor' => $desc->bookGr->consignor_id ? $desc->bookGr->consignor : ['id'=> null, 'party_name'=>$desc->bookGr->consignor_name],
                    'consignee' => $desc->bookGr->consignee_id ? $desc->bookGr->consignee : ['id'=> null, 'party_name'=>$desc->bookGr->consignee_name],
                    'source' => $desc->bookGr->source_id ? $desc->bookGr->source : ['id'=> null, 'station_name'=>$desc->bookGr->source_name],
                    'destination' => $desc->bookGr->destination ? $desc->bookGr->destination : ['id'=> null, 'station_name'=>$desc->bookGr->destination_name] ,
                    'total_pkg' => $desc->total_pkg,
                    'total_material_weight' => $desc->total_material_weight,
                    'total_chg_qty' => $desc->total_chg_qty,
                    'total_desc_value' => $desc->total_desc_value,
                    'total_invoice_value' => $desc->total_invoice_value,
                    'freight' => $desc->freight,
                    'gr_chg' => $desc->gr_chg,
                    'gr_desc' => json_decode($desc->gr_desc),
                    'pod_id' => $desc->pod ? $desc->pod_id : null,
                    'pod_number' =>  $desc->pod ? $desc->pod->pod_number:null,
                    'load_detention' => $desc->load_detention,
                    'unload_detention' => $desc->unload_detention,
                    'payment_dynamic_fields' => json_decode($desc->payment_dynamic_fields)??[],
                    'pod_payment_dynamic_fields' => json_decode($desc->pod_payment_dynamic_fields)??[],
                ];
            }
        
            if ($desc->selection_type == 'Loading Slip') {
                $data += [
                    'ls_id' => $desc->ls_id,
                    'ls_number' => $desc->ls_number,
                    'ls_date' => $desc->loading_slip->ls_date,
                    'branch' => $desc->loading_slip->branch,
                    'vehicle' => $desc->loading_slip->vehicle,
                    'source' => $desc->loading_slip->source,
                    'destination' => $desc->loading_slip->destination,
                    'freight' => $desc->loading_slip->freight,
                    'munshiana' => $desc->loading_slip->munshiana,
                    'dalla' => $desc->loading_slip->dalla,
                    'advance_amt' => $desc->loading_slip->advance_amt,
                    'balance_received_in_amt' => $desc->loading_slip->balance_received_in_amt,
                    'diesel_amt' => $desc->loading_slip->diesel_amt,
                    'payment_and_collection' => $desc->loading_slip->payment_and_collection,
                    'labour' => $desc->loading_slip->labour,
                    'detention' => $desc->loading_slip->detention,
                    'unload_detention' => $desc->loading_slip->unload_detention,
                    'balance_receivable' => $desc->loading_slip->balance_receivable,
                ];
            }

            if ($desc->selection_type == 'Supplementary') {
                $data += [
                    'supplementary_desc' => json_decode($desc->supplementary_desc),
                ];
            }
        
            return $data;
        });        

        return view('docs.bill_print')->with(['bill' => $bill,'company'=>$this->company]);
    }

    public function open_ls_bill_print(Request $request){
        $this->initialize($request->cc);
        $bill = Bill::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'acc_party_branch:id,branch_name',
            'acc_party.party:id,account_id,party_name,address1,address2,state,tele_number,gst_number',
            'income:id,account_name',

            'bill_desc',
            'bill_desc.bookGr:id,gr_number,gr_date,consignor_id,consignee_id,source_id,destination_id,consignor_name,consignee_name,source_name,destination_name',
            'bill_desc.bookGr.consignor:id,party_name',
            'bill_desc.bookGr.consignee:id,party_name',
            'bill_desc.bookGr.source:id,station_name',
            'bill_desc.bookGr.destination:id,station_name',
            'bill_desc.pod:id,pod_number',
            
            'bill_desc.bookGr.vehicle:id,vehicle_number,vehicle_type',

            'bill_desc.loading_slip'
        ])->find($request->pk);


        $bill->bill_desc->transform(function ($desc) {
            $data = [
                'bill_desc_id' => $desc->id,
                'bill_id' => $desc->bill_id,
                'total_without_gst' => $desc->total_without_gst,
                'igst_amount' => $desc->igst_amount,
                'cgst_amount' => $desc->cgst_amount,
                'sgst_amount' => $desc->sgst_amount,
                'rounded_off' => $desc->rounded_off,
                'grand_total' => $desc->grand_total,
            ];

            
        
            if ($desc->selection_type == 'GR') {
                $data += [
                    'gr_id' => $desc->gr_id,
                    'gr_number' => $desc->gr_number,
                    'gr_date' => $desc->bookGr->gr_date,
                    'consignor' => $desc->bookGr->consignor_id ? $desc->bookGr->consignor : ['id'=> null, 'party_name'=>$desc->bookGr->consignor_name],
                    'consignee' => $desc->bookGr->consignee_id ? $desc->bookGr->consignee : ['id'=> null, 'party_name'=>$desc->bookGr->consignee_name],
                    'source' => $desc->bookGr->source_id ? $desc->bookGr->source : ['id'=> null, 'station_name'=>$desc->bookGr->source_name],
                    'destination' => $desc->bookGr->destination ? $desc->bookGr->destination : ['id'=> null, 'station_name'=>$desc->bookGr->destination_name] ,
                    'total_pkg' => $desc->total_pkg,
                    'total_material_weight' => $desc->total_material_weight,
                    'total_chg_qty' => $desc->total_chg_qty,
                    'total_desc_value' => $desc->total_desc_value,
                    'total_invoice_value' => $desc->total_invoice_value,
                    'freight' => $desc->freight,
                    'gr_chg' => $desc->gr_chg,
                    'gr_desc' => json_decode($desc->gr_desc),
                    'pod_id' => $desc->pod ? $desc->pod_id : null,
                    'pod_number' =>  $desc->pod ? $desc->pod->pod_number:null,
                    'load_detention' => $desc->load_detention,
                    'unload_detention' => $desc->unload_detention,
                    'payment_dynamic_fields' => json_decode($desc->payment_dynamic_fields)??[],
                    'pod_payment_dynamic_fields' => json_decode($desc->pod_payment_dynamic_fields)??[],
                ];
            }
        
            if ($desc->selection_type == 'Loading Slip') {
                $data += [
                    'ls_id' => $desc->ls_id,
                    'ls_number' => $desc->ls_number,
                    'total_pkg' => $desc->loading_slip->pkg,
                    'rate' => $desc->loading_slip->rate,
                    'ls_date' => $desc->loading_slip->ls_date,
                    'branch' => $desc->loading_slip->branch,
                    'vehicle' => $desc->loading_slip->vehicle,
                    'source' => $desc->loading_slip->source,
                    'destination' => $desc->loading_slip->destination,
                    'freight' => $desc->loading_slip->freight,
                    'munshiana' => $desc->loading_slip->munshiana,
                    'dalla' => $desc->loading_slip->dalla,
                    'advance_amt' => $desc->loading_slip->advance_amt,
                    'balance_received_in_amt' => $desc->loading_slip->balance_received_in_amt,
                    'diesel_amt' => $desc->loading_slip->diesel_amt,
                    'payment_and_collection' => $desc->loading_slip->payment_and_collection,
                    'labour' => $desc->loading_slip->labour,
                    'detention' => $desc->loading_slip->detention,
                    'unload_detention' => $desc->loading_slip->unload_detention,
                    'balance_receivable' => $desc->loading_slip->balance_receivable,
                ];
            }

            if ($desc->selection_type == 'Supplementary') {
                $data += [
                    'supplementary_desc' => json_decode($desc->supplementary_desc),
                ];
            }
        
            return $data;
        });        

        return view('docs.ls_bill_print')->with(['bill' => $bill,'company'=>$this->company]);
    }


    protected function initialize($companyCode)
    {
        $this->connectToTheDB($companyCode);

        $this->company = Company::where('company_code', $companyCode)->first();
    }
    
    protected function connectToTheDB($company){
        try {
            // Update the 'database' config in the 'dynamic' connection to point to the current company's database
            config(['database.connections.dynamic.database' => config('app.SERVER_PREFIX') .strtolower($company)]);
        
            // Reconnect to the updated database connection
            DB::connection('dynamic')->reconnect();
        
            DB::setDefaultConnection('dynamic');

        } catch (\Exception $e) {
            // Handle the exception here, for example:
            return response()->json(['message' => 'Failed to update dynamic database connection'], 500);
        }
        
    }
}
