<?php

namespace App\Http\Controllers\Z_Core;

use App\Http\Controllers\Controller;
use App\Models\Apis\ApiCounter;
use App\Models\Booking\BookGr;
use App\Models\Booking\EwayGr;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\Z_Core\EditLog;
use Carbon\Carbon;
use Illuminate\Http\Request;

class DashboardController extends Controller
{
    public function index(Request $request)
    {
        // Get total records for today
        $totalRecordsToday = LoadingSlip::whereDate('ls_date', today())->where('fin_year',$request->current_year)->count();

        // Get total records for the current month
        $totalRecordsThisMonth = LoadingSlip::whereMonth('ls_date', now())->where('fin_year',$request->current_year)->count();

        $totalRecordsLastMonth = LoadingSlip::whereMonth('ls_date', now()->subMonth())->where('fin_year',$request->current_year)->count();


        $totalUpdatesToday = EditLog::whereDate('created_at', today())->where('operation', 'updated')->where('fin_year',$request->current_year)->count();
        $totalDeletesToday = EditLog::whereDate('created_at', today())->where('operation', 'deleted')->where('fin_year',$request->current_year)->count();

        $pendingBillsGr = BookGr::where('pay_mode','TBB')->where('freight_bill',0)->orWhere('way_exp_bill',0)->where('fin_year',$request->current_year)->count();
        $pendingBillsLs = LoadingSlip::where('bill_status',0)->where('fin_year',$request->current_year)->count();



        // I WANT THEM FOR DISTINCT API_NAMES
        $api_data = ApiCounter::select(
            'api_name',
            \DB::raw('SUM(CASE WHEN DATE(date) = CURDATE() THEN api_count ELSE 0 END) as today'),
            \DB::raw('SUM(api_count) as total')
        )
        ->groupBy('api_name')
        ->get();

        // Your other logic for the index method...

        return response()->json([
            'totalRecordsToday' => $totalRecordsToday,
            'totalRecordsThisMonth' => $totalRecordsThisMonth,
            'totalRecordsLastMonth' => $totalRecordsLastMonth,
            'totalUpdatesToday' => $totalUpdatesToday,
            'totalDeletesToday' => $totalDeletesToday,
            'pendingBillsGr' => $pendingBillsGr,
            'pendingBillsLs' => $pendingBillsLs,
            'api_data' => $api_data,


        ]);
    }

    public function get_eway_reminders(Request $request)
    {
        $days = $request->days;
        // Calculate the date 15 days from now
        $allowedTillDate = Carbon::now()->addDays($days);

        // Get all the records that are not updated since 15 days, whose pod has not been made
        $eway = EwayGr::select('id','gr_id','gr_number','eway_number','eway_exp_date')->where('eway_exp_date', '<=', $allowedTillDate)
                ->whereHas('bookGr', function ($query) {
                    $query->doesntHave('pod_desc');
                })
                ->where('is_delivered',0)
                ->where('fin_year',$request->current_year)
                ->get();

        // ADD `field_differenct` to each record
        foreach ($eway as $key => $value) {
            $value->field_difference = $allowedTillDate->diffInDays($value->eway_exp_date);
        }


        return response()->json($eway);
    }


}
