<?php

namespace App\Http\Controllers\Z_Core;

use App\Http\Controllers\Controller;
use App\Models\Z_Core\EditLog;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class EditLogController extends Controller
{
    public function index(Request $request){
        if(!$request->user->power){
            return response()->json(['Permission Denied For The Action'],401);
        }
        //FETCH AND SEND ALL
        $edit_log = EditLog::select('edit_logs.*' ,DB::raw('DATE_FORMAT(edit_logs.created_at, "%Y-%m-%d %h:%i %p") as formatted_created_at'), 'users.username')->leftJoin('users', 'edit_logs.user_id', '=', 'users.id')->where('fin_year',$request->current_year)->orderBy('edit_logs.id','desc')->get();
        return response()->json($edit_log);
    }

    public function show(Request $request,$pk){
        if(!$request->user->power){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCH AND SEND
        $edit_log = EditLog::leftJoin('users', 'edit_logs.user_id', '=', 'users.id')
                ->select('edit_logs.*', DB::raw('DATE_FORMAT(edit_logs.created_at, "%Y-%m-%d %h:%i %p") as formatted_created_at'), 'users.username')
                ->where('fin_year',$request->current_year)
                ->find($pk);
        return response()->json($edit_log);
    }

    public function datatable(Request $request){
        if(!$request->user->power){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;
        $filterValue = $request->filter;

        $query = EditLog::select('edit_logs.*' ,DB::raw('DATE_FORMAT(edit_logs.created_at, "%d-%m-%Y %h:%i %p") as formatted_created_at'), 'users.username')->leftJoin('users', 'edit_logs.user_id', '=', 'users.id')
                ->where('fin_year',$request->current_year)->whereIn('operation',['updated','deleted'])->orderBy($column, $dir);

        if ($filterValue) {
            $query->where(function ($query) use ($filterValue) {
                if($filterValue['operation'] != 'all')
                    $query->where('operation', $filterValue['operation'] );
            });
        }

        if($searchValue){
            $query->where(function($query) use ($searchValue) {
                $query->where('edit_logs.created_at', 'like', '%' . $searchValue . '%')
                ->orWhere('username', 'like', '%' . $searchValue . '%')
                ->orWhere('table_name', 'like', '%' . $searchValue . '%')
                ->orWhere('ref_value', 'like', '%' . $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }
}
