<?php

namespace App\Http\Controllers\Z_Core;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Models\Account\Voucher;
use App\Models\Account\VoucherAccDetails;
use App\Models\Account\VoucherDesc;
use App\Models\Master\Branch;
use App\Models\Master\Vehicle;
use App\Models\Settings\GeneralSettings;
use Carbon\Carbon;
use DateTime;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class ExcelUploadController extends Controller
{
    use CommonTrait;

    public function upload_fastag(Request $request){

        $required_fields = ['date','vehicle_number','amount'];



        $FASTAG_EXPENSE = $this->get_acc_id('FASTAG EXPENSE');
        $PAYMENT_FROM = $this->get_acc_id('AXIS4155');

        if(!$FASTAG_EXPENSE || !$PAYMENT_FROM){
            return response('The Mandatory FASTAG or Payment Account not found',500);
        }

        $excelData = $request->excelData;

        // Check if the first row of the Excel data array contains all the required fields
        foreach ($required_fields as $field) {
            // Check if the required field is missing in the first row
            if (!in_array($field, $excelData[0])) {
                // Handle the case where a required field is missing
                return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.'],500);
            }
        }


        $branch_to_use = 'ALA';

        $check_branch = Branch::where('branch_code', $branch_to_use)->first();
        if(!$check_branch){
            return response(['message'=>'Contact Admin To Set A Branch To import data']);
        }
        
        $fin_yr_wise = GeneralSettings::where('setting_name','voucher_number_fin_yr_wise')->where('value',1)->exists();

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

                $rand = rand();
                for ($i = 1; $i < count($excelData); $i++) {
                    $item = $excelData[$i];

                    

                    $required_err = false;
                    foreach ($required_fields as $field) {
                        // Check if the required field is missing in the first row
                        
                        if (!isset($item[$field])) {
                            // Handle the case where a required field is missing
                           $required_err = true;
                           // return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.', 'item'=>$item],500);
                        }
                    }
                    
                    if($required_err==true || !$item['amount']){
                        continue;
                    }


                    $max = Voucher::where('branch_id',$check_branch->id)->where('voucher_type', 'SYSTEM')->where('voucher_name','Journal')->where('fin_yr_wise',$fin_yr_wise)->max('max_number');
                
                    if($fin_yr_wise){
                        $year_stamp = $this->get_short_name($request->current_year);
                        $voucher_number = $check_branch->branch_code.'/'.$year_stamp.'/'.sprintf('%06d', $max+1);
                    }
                    else{
                        $voucher_number = $check_branch->branch_code.'-'.sprintf('%06d', $max+1);
                    }

                    
                    // SAVING A new Voucher
                    $voucher = new Voucher;
                    $voucher->branch_id = $check_branch['id'];
                    $voucher->voucher_type = 'SYSTEM';
                    $voucher->fin_yr_wise = $fin_yr_wise;
                    $voucher->max_number = $max+1;
                    $voucher->voucher_name = 'Journal';

                    $voucher->voucher_number = $voucher_number;
                    $voucher->voucher_date = DateTime::createFromFormat('d-m-Y', $item['date'])->format('Y-m-d');
                    $voucher->voucher_time = Carbon::now();

                    $desc = "";

                    if(isset($item['toll_id'])){
                        $desc.='Toll Id : '.$item['toll_id'];
                    }
                    if(isset($item['transaction_id'])){
                        $desc.='Transaction Id : '.$item['transaction_id'];
                    }
                    if(isset($item['description'])){
                        $desc.='Description: '.$item['description'];
                    }


                    $voucher->narration = $desc;
                    $voucher->total_dr_amount = $item['amount'];
                    $voucher->total_cr_amount = $item['amount'];
                    $voucher->fin_year = $request->current_year;
                    
                    $voucher->rndm =  $rand;
                    $voucher->created_by = $request->user->id;
                    $voucher->save(); 
                    // new Voucher SAVED



                        $cr_v_acc_detail = new VoucherAccDetails();
                        $cr_v_acc_detail->voucher_id = $voucher->id;
                        $cr_v_acc_detail->voucher_name = $voucher->voucher_name;
                        $cr_v_acc_detail->voucher_number = $voucher->voucher_number;
                        $cr_v_acc_detail->account_id = $PAYMENT_FROM;
                        $cr_v_acc_detail->nature = 'cr';
                        $cr_v_acc_detail->cr_amount = $item['amount'];
                        $cr_v_acc_detail->fin_year = $voucher->fin_year;
                        $cr_v_acc_detail->rndm =  $rand;
                        $cr_v_acc_detail->save();

                        $v_acc_detail = new VoucherAccDetails();
                        $v_acc_detail->voucher_id = $voucher->id;
                        $v_acc_detail->voucher_name = $voucher->voucher_name;
                        $v_acc_detail->voucher_number = $voucher->voucher_number;
                        $v_acc_detail->account_id = $FASTAG_EXPENSE;
                        $v_acc_detail->nature = 'dr';
                        $v_acc_detail->against = 'VEHICLE_COST_CENTER';
                        $v_acc_detail->dr_amount = $item['amount'];
                        $v_acc_detail->fin_year = $voucher->fin_year;
                        $v_acc_detail->rndm =  $rand;
                        $v_acc_detail->save();




                        if ($v_acc_detail['against']) {
                                $vou_desc = new VoucherDesc();
                                $vou_desc->voucher_id = $voucher->id;
                                $vou_desc->voucher_name = $voucher->voucher_name;
                                $vou_desc->voucher_number = $voucher->voucher_number;
                                $vou_desc->v_acc_det_id = $v_acc_detail->id;
                                $vou_desc->against = 'VEHICLE_COST_CENTER';


                                $vehicle = Vehicle::select('id','vehicle_number')->where('vehicle_number', $item['vehicle_number'])->first();
                              
                                if (!$vehicle) {

                                    // Handle the case where a required field is missing
                                    return response()->json(['status' => false, 'message' => 'Vehicle not exist  is missing.', 'item'=>$item],500);
                                }

                                $vou_desc->against_id = $vehicle['id'];
                                $vou_desc->against_number = $vehicle['vehicle_number'];
                                $vou_desc->received_amt = $item['amount'];
                                $vou_desc->receipt_amt = $item['amount'];
                                $vou_desc->fin_year = $voucher->fin_year;
                                $vou_desc->rndm =  $rand;
                                $vou_desc->save();
                                
                        }
                    
                }

            // TRANSACTIONS COMMITTED
            DB::commit();

                
            return response()->json(['success'=>true], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
        

        

        

    }


    protected function get_max_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

}
