<?php

namespace App\Models\Account;

use App\Models\Account\AccountTransaction;
use App\Models\Account\Balances;
use App\Models\Account\Group;
use App\Models\Account\Subgroup;
use App\Models\Master\Branch;
use App\Models\Master\Employee;
use App\Models\Master\Party;
use App\Models\Master\Vehicle;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Account extends Model
{
    use HasFactory;
    
    // Account Belongs To A Group
    public function group()
    {
        return $this->belongsTo(Group::class);
    }
    
    // Account Can Have Many Transactions
    public function transactions()
    {
        return $this->hasMany(AccountTransaction::class);
    }
    
    // Account Can Have Many Balances ( Many Because Multiple Years. One entry for each year. )
    public function balances()
    {
        return $this->hasMany(Balances::class);
    }

    public function voucher_acc_details()
    {
        return $this->hasMany(VoucherAccDetails::class);
    }


    public function party()
    {
        // The second parameter is the foreign key on the parties table
        // The third parameter is the primary key on the accounts table
        return $this->hasOne(Party::class, 'account_id', 'id');
    }

    public function branch()
    {
        // The second parameter is the foreign key on the parties table
        // The third parameter is the primary key on the accounts table
        return $this->hasOne(Branch::class, 'account_id', 'id');
    }

    public function vehicle()
    {
        // The second parameter is the foreign key on the parties table
        // The third parameter is the primary key on the accounts table
        return $this->hasOne(Vehicle::class, 'account_id', 'id');
    }

    public function employee()
    {
        // The second parameter is the foreign key on the parties table
        // The third parameter is the primary key on the accounts table
        return $this->hasOne(Employee::class, 'account_id', 'id');
    }
    
}
