<?php

namespace App\Models\Booking;

use App\Models\Account\Account;
use App\Models\Account\VoucherDesc;
use App\Models\Bill\BillDesc;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\FleetOwner\DriverSettlementDesc;
use App\Models\FleetOwner\LorryHireDesc;
use App\Models\GoodsMove\ChallanDesc;
use App\Models\GoodsMove\ChallanReceiptDesc;
use App\Models\Master\Branch;
use App\Models\Master\Employee;
use App\Models\Master\Party;
use App\Models\Master\PartyBranches;
use App\Models\Master\Station;
use App\Models\Master\Vehicle;
use App\Models\Pod\PodDesc;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class BookGr extends Model
{
    use HasFactory;

    public function eway_details()
    {
        return $this->hasMany(EwayGr::class,'gr_id');
    }

    public function package_position()
    {
        return $this->hasMany(PackagePosition::class,'gr_id');
    }

    public function diesel_receipt()
    {
        return $this->hasMany(DieselReceipt::class,'booking_id')->where('against','GR');
    }
    

    public function voucher_desc()
    {
        return $this->hasMany(VoucherDesc::class,'against_number','gr_number');
    }

    public function challan_desc()
    {
        return $this->hasMany(ChallanDesc::class,'gr_id');
    }

    public function challan_rec_desc()
    {
        return $this->hasMany(ChallanReceiptDesc::class,'gr_id');
    }

    public function lorry_hire_desc()
    {
        return $this->hasMany(LorryHireDesc::class,'against_id','id');
    }

    public function pod_desc()
    {
        return $this->hasOne(PodDesc::class, 'gr_id');
    }

    public function gr_desc()
    {
        return $this->hasMany(GrDesc::class,'gr_id');
    }

    public function bill_desc()
    {
        return $this->hasMany(BillDesc::class,'gr_id');
    }

    public function branch()
    {
        return $this->belongsTo(Branch::class, 'branch_id');
    }

    public function source()
    {
        return $this->belongsTo(Station::class, 'source_id');
    }

    public function destination()
    {
        return $this->belongsTo(Station::class, 'destination_id');
    }

    public function consignor()
    {
        return $this->belongsTo(Party::class, 'consignor_id');
    }

    public function consignor_branch()
    {
        return $this->belongsTo(PartyBranches::class, 'consignor_branch_id');
    }

    public function consignee()
    {
        return $this->belongsTo(Party::class, 'consignee_id');
    }

    public function consignee_branch()
    {
        return $this->belongsTo(PartyBranches::class, 'consignee_branch_id');
    }

    public function vehicle()
    {
        return $this->belongsTo(Vehicle::class, 'vehicle_id');
    }

    public function driver()
    {
        return $this->belongsTo(Employee::class, 'driver_id');
    }

    public function acc_party()
    {
        return $this->belongsTo(Account::class, 'acc_party_id');
    }

    public function income()
    {
        return $this->belongsTo(Account::class, 'income_account_id');
    }
    
    public function acc_party_branch()
    {
        return $this->belongsTo(PartyBranches::class, 'acc_party_branch_id');
    }

    public function d_settlement_desc()
    {
        return $this->hasMany(DriverSettlementDesc::class,'against_id','id');
    }


}
