<?php

namespace App\Models\Master;

use App\Models\Account\Account;
use App\Models\User;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Party extends Model
{
    use HasFactory;
    
    // PARTY CAN HAVE MANY USERS (ONE TO MANY)
    public function users()
    {
        return $this->hasMany(User::class);
    }
    
    // PARTY CAN HAVE MANY BRANCHES (ONE TO MANY)
    public function party_branches()
    {
        return $this->hasMany(PartyBranches::class);
    }
    public function account()
    {
        // The second parameter is the foreign key on the parties table
        // The third parameter is the primary key on the accounts table
        return $this->belongsTo(Account::class, 'account_id', 'id');
    }
    
    // PARTY CAN HAVE MANY OPENING (ONE TO MANY)
    public function party_opening_details()
    {
        return $this->hasMany(PartyOpeningDetails::class);
    }
    
    // PARTY CAN OWN MANY VEHICLES (ONE TO MANY)
    public function vehicles()
    {
        return $this->hasMany(Vehicle::class);
    }
    
    // PARTY CAN HAVE MANY BOOKS ISSUED TO THEM (ONE TO MANY)
    public function issued_books()
    {
        return $this->hasMany(IssueBooks::class);
    }
    
    // PARTY CAN BELONG TO A PARTICULAR BRANCH
    public function branch()
    {
        return $this->belongsTo(Branch::class, 'branch_id', 'id');
    }
    
    // Can have a preset material
    public function material()
    {
        return $this->belongsTo(Material::class);
    }

    public function preset_on_account()
    {
        return $this->belongsTo(Account::class, 'preset_on_account_id', 'id');
    }



}
