<?php

use App\Http\Controllers\Accounts\AccountController;
use App\Http\Controllers\Accounts\GroupController;
use App\Http\Controllers\Accounts\VoucherController;
use App\Http\Controllers\Api\EinvoiceController;
use App\Http\Controllers\Api\EwayController;
use App\Http\Controllers\Api\WhatsappController;
use App\Http\Controllers\Auth\AuthController;
use App\Http\Controllers\Auth\UserController;
use App\Http\Controllers\Bill\BillController;
use App\Http\Controllers\Bill\CoverLetterController;
use App\Http\Controllers\Bill\CreditNoteController;
use App\Http\Controllers\Booking\GrController;
use App\Http\Controllers\FleetOwner\DieselReceiptController;
use App\Http\Controllers\FleetOwner\DriverSettlementController;
use App\Http\Controllers\FleetOwner\LoadingSlipController;
use App\Http\Controllers\FleetOwner\LorryHireController;
use App\Http\Controllers\GoodsMove\ChallanController;
use App\Http\Controllers\GoodsMove\ChallanReceiptController;
use App\Http\Controllers\GoodsMove\UnloadingController;
use App\Http\Controllers\HR\AttendanceController;
use App\Http\Controllers\HR\HolidayController;
use App\Http\Controllers\HR\SalaryController;
use App\Http\Controllers\Inventory\BrandController;
use App\Http\Controllers\Inventory\ProductCategoryController;
use App\Http\Controllers\Inventory\ProductController;
use App\Http\Controllers\Inventory\PurchaseController;
use App\Http\Controllers\Inventory\StockIssueController;
use App\Http\Controllers\Items\CategoryController;
use App\Http\Controllers\Items\ItemController;
use App\Http\Controllers\Leads\LeadController;
use App\Http\Controllers\Leads\QuotationController;
use App\Http\Controllers\Master\BranchController;
use App\Http\Controllers\Master\EmployeeController;
use App\Http\Controllers\Master\IssueBooksController;
use App\Http\Controllers\Master\LateDeductionController;
use App\Http\Controllers\Master\MaterialController;
use App\Http\Controllers\Master\PackageMethodController;
use App\Http\Controllers\Master\PartyController;
use App\Http\Controllers\Master\PriceListController;
use App\Http\Controllers\Master\RouteController;
use App\Http\Controllers\Master\StationController;
use App\Http\Controllers\Master\UnitController;
use App\Http\Controllers\Master\VehicleController;
use App\Http\Controllers\Master\VehicleTypeController;
use App\Http\Controllers\Pod\PodController;
use App\Http\Controllers\Settings\SettingsController;
use App\Http\Controllers\Z_Core\BackupController;
use App\Http\Controllers\Z_Core\DashboardController;
use App\Http\Controllers\Z_Core\DocumentController;
use App\Http\Controllers\Z_Core\EditLogController;
use App\Http\Controllers\Z_Core\EmailController;
use App\Http\Controllers\Z_Core\ExcelUploadController;
use App\Http\Controllers\Z_Core\GstController;
use Carbon\Carbon;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

// Route::middleware('auth:sanctum')->get('/user', function (Request $request) {
//     return $request->user();
// });


Route::post('/auth/login', [AuthController::class, 'login']);

Route::get('/current-datetime', function () {
    $currentDateTime = Carbon::now();
    
    return response()->json([
        'date' => $currentDateTime->toDateString(),
        'time' => $currentDateTime->format('H:i')
    ]);
});

Route::group(['middleware'=>'custom.jwt'], function ($routes){
    Route::post('/auth/logout', [AuthController::class, 'logout']);
    
    Route::get('/auth/user', [AuthController::class, 'user']);
    Route::post('/auth/refresh', [AuthController::class, 'refresh']);

    // ~~~~~~~~~~~~~~~~~~~~ DOCUMENT ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~
    Route::post('/document/upload', [DocumentController::class, 'upload']);
    // ~~~~~~~~~~~~~~~~~~~~ END DOCUMENT ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // ~~~~~~~~~~~~~~~~~~~~ EXCEL ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~
    Route::post('/excel/upload/fastag', [ExcelUploadController::class, 'upload_fastag']);
    // ~~~~~~~~~~~~~~~~~~~~ END EXCEL ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // ~~~~~~~~~~~~~~~~~~~~ GST ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~
    Route::get('/fetch_state_for_gst/{gst_number}', [GstController::class, 'getStateName']);
    Route::get('/gst/get_gst_data', [GstController::class, 'get_gst_data']);
    // ~~~~~~~~~~~~~~~~~~~~ END GST ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~


    // ~~~~~~~~~~~~~~~~~~~~ START ACCOUNTS ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~
    // ACCOUNT ROUTES
    Route::get('/accounts', [AccountController::class, 'index']); 
    Route::get('/datatable/account', [AccountController::class, 'datatable']); 
    Route::post('/account', [AccountController::class, 'store']);
    Route::get('/autocomplete/account/', [AccountController::class, 'autocomplete']);
    Route::get('/account/{pk}', [AccountController::class, 'show']);
    Route::put('/account/{pk}', [AccountController::class, 'update']);
    Route::delete('/account/{pk}', [AccountController::class, 'destroy']);
    Route::get('/check/unique/account', [AccountController::class, 'check_unique']);

    // GROUPS Routes
    Route::get('/groups', [GroupController::class, 'index']); 
    Route::get('/datatable/group', [GroupController::class, 'datatable']); 
    Route::post('/group', [GroupController::class, 'store']);
    Route::get('/autocomplete/group/', [GroupController::class, 'autocomplete']);
    Route::get('/group/{pk}', [GroupController::class, 'show']);
    Route::delete('/group/{pk}', [GroupController::class, 'destroy']);
    Route::get('/check/unique/group', [GroupController::class, 'check_unique']);
    Route::get('/primary/groups', [GroupController::class, 'get_primary_groups']);

    // Voucher Routes
    Route::get('/voucher', [VoucherController::class, 'index']); 
    Route::get('/datatable/voucher', [VoucherController::class, 'datatable']); 
    Route::post('/voucher', [VoucherController::class, 'store']);
    Route::get('/autocomplete/voucher/', [VoucherController::class, 'autocomplete']);
    Route::get('/voucher/{pk}', [VoucherController::class, 'show']);
    Route::put('/voucher/{pk}', [VoucherController::class, 'update']);
    Route::delete('/voucher/{pk}', [VoucherController::class, 'destroy']);
    Route::get('/check/unique/voucher', [VoucherController::class, 'check_unique']);
    Route::get('/get/next/voucher_number', [VoucherController::class, 'get_next_voucher_number']);
    Route::get('/autocomplete/voucher/against/{selection}', [VoucherController::class, 'autocomplete_for_against']);
    Route::get('/voucher/fetch/against/details/{selection}/{pk}', [VoucherController::class, 'fetch_against_details']);
    Route::post('/reports/voucher_report', [VoucherController::class, 'get_voucher_report']);  


    // ~~~~~~~~~~~~~~~~~~~~ END ACCOUNTS ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    
    // ~~~~~~~~~~~~~~~~~~~~ START ADMIN ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    // USER ROUTES
    Route::get('/user', [UserController::class, 'index']); 
    Route::get('/datatable/user', [UserController::class, 'datatable']); 
    Route::post('/user', [UserController::class, 'store']);
    Route::get('/autocomplete/user/', [UserController::class, 'autocomplete']);
    Route::get('/user/{pk}', [UserController::class, 'show']);
    Route::put('/user/{pk}', [UserController::class, 'update']);
    Route::delete('/user/{pk}', [UserController::class, 'destroy']);
    Route::get('/check/unique/user', [UserController::class, 'check_unique']);
    Route::put('/user/reset_password/{pk}', [UserController::class, 'reset_password']);
    
    // EDIT LOG ROUTES
    Route::get('/edit_log', [EditLogController::class, 'index']); 
    Route::get('/datatable/edit_log', [EditLogController::class, 'datatable']); 
    Route::get('/edit_log/{pk}', [EditLogController::class, 'show']);

    Route::post('/send-email', [EmailController::class, 'sender']);

    Route::post('/send-whatsapp', [WhatsappController::class, 'sender']);

    // DB BACKUP ROUTES
    Route::get('/backup', [BackupController::class, 'download_backup']);


    // ~~~~~~~~~~~~~~~~~~~~ END ADMIN ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // ~~~~~~~~~~~~~~~~~~~~ START MASTER ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // BRANCH ROUTES
    Route::get('/branch', [BranchController::class, 'index']); 
    Route::get('/datatable/branch', [BranchController::class, 'datatable']); 
    Route::post('/branch', [BranchController::class, 'store']);
    Route::get('/autocomplete/branch/', [BranchController::class, 'autocomplete']);
    Route::get('/branch/{pk}', [BranchController::class, 'show']);
    Route::put('/branch/{pk}', [BranchController::class, 'update']);
    Route::delete('/branch/{pk}', [BranchController::class, 'destroy']);
    Route::get('/check/unique/branch', [BranchController::class, 'check_unique']);
    Route::get('/branch/get_branch_station/{pk}', [BranchController::class, 'get_branch_station']);
    Route::get('/get/custom_fields/branch', [BranchController::class, 'get_custom_fields']);
    Route::get('/branch/delivery_commission_details/{pk}', [BranchController::class, 'delivery_commission_details']);
    Route::post('/reports/branch', [BranchController::class, 'get_branch_report']);
    
    // PARTY ROUTES
    Route::get('/party', [PartyController::class, 'index']); 
    Route::get('/datatable/party', [PartyController::class, 'datatable']); 
    Route::post('/party', [PartyController::class, 'store']);
    Route::get('/autocomplete/party/', [PartyController::class, 'autocomplete']);
    Route::get('/autocomplete/party_branches/', [PartyController::class, 'autocomplete_for_party_branches']);
    Route::get('/party/{pk}', [PartyController::class, 'show']);
    Route::put('/party/{pk}', [PartyController::class, 'update']);
    Route::delete('/party/{pk}', [PartyController::class, 'destroy']);
    Route::get('/check/unique/party', [PartyController::class, 'check_unique']);
    Route::get('/get/custom_fields/party', [PartyController::class, 'get_custom_fields']);
    Route::get('/get/custom_fields/party_branch', [PartyController::class, 'get_custom_fields_for_party_branches']);
    Route::post('/reports/party', [PartyController::class, 'get_party_report']);
    Route::get('/party/from/gst', [PartyController::class, 'get_party_from_gst_number']); 

    // VEHICLE ROUTES
    Route::get('/vehicle', [VehicleController::class, 'index']); 
    Route::get('/datatable/vehicle', [VehicleController::class, 'datatable']); 
    Route::post('/vehicle', [VehicleController::class, 'store']);
    Route::get('/autocomplete/vehicle/', [VehicleController::class, 'autocomplete']);
    Route::get('/vehicle/{pk}', [VehicleController::class, 'show']);
    Route::get('/vehicle/get_autorised_driver/{pk}', [VehicleController::class, 'get_autorised_driver']);
    Route::put('/vehicle/{pk}', [VehicleController::class, 'update']);
    Route::delete('/vehicle/{pk}', [VehicleController::class, 'destroy']);
    Route::get('/check/unique/vehicle', [VehicleController::class, 'check_unique']);
    Route::get('/get/vehicle/reminders', [VehicleController::class, 'get_vehicle_reminders']);
    Route::post('/reports/vehicle', [VehicleController::class, 'get_vehicle_report']);
    
    // EMPLOYEE ROUTES
    Route::get('/employee', [EmployeeController::class, 'index']); 
    Route::get('/datatable/employee', [EmployeeController::class, 'datatable']); 
    Route::post('/employee', [EmployeeController::class, 'store']);
    Route::get('/autocomplete/employee/', [EmployeeController::class, 'autocomplete']);
    Route::get('/employee/{pk}', [EmployeeController::class, 'show']);
    Route::put('/employee/{pk}', [EmployeeController::class, 'update']);
    Route::delete('/employee/{pk}', [EmployeeController::class, 'destroy']);
    Route::get('/check/unique/employee', [EmployeeController::class, 'check_unique']);
    Route::get('/check/black_list/employee', [EmployeeController::class, 'check_black_list']);
    Route::get('/get/next/emp_id', [EmployeeController::class, 'get_next_emp_id']);
    Route::post('/reports/employee', [EmployeeController::class, 'get_employee_report']);
    Route::get('/autocomplete/driver_for_driver_stl', [EmployeeController::class, 'autocomplete_for_driver_stl']);
    
    // STATION ROUTES
    Route::get('/station', [StationController::class, 'index']); 
    Route::get('/datatable/station', [StationController::class, 'datatable']); 
    Route::post('/station', [StationController::class, 'store']);
    Route::get('/autocomplete/station/', [StationController::class, 'autocomplete']);
    Route::get('/station/{pk}', [StationController::class, 'show']);
    Route::put('/station/{pk}', [StationController::class, 'update']);
    Route::delete('/station/{pk}', [StationController::class, 'destroy']);
    Route::get('/check/unique/station', [StationController::class, 'check_unique']);
    
    // ISSUE BOOKS ROUTES
    Route::get('/issue_books', [IssueBooksController::class, 'index']); 
    Route::get('/datatable/issue_books', [IssueBooksController::class, 'datatable']); 
    Route::post('/issue_books', [IssueBooksController::class, 'store']);
    Route::get('/issue_books/{pk}', [IssueBooksController::class, 'show']);
    Route::delete('/issue_books/{pk}', [IssueBooksController::class, 'destroy']);
    Route::get('/check/unique/issue_books', [IssueBooksController::class, 'check_unique']);
    Route::get('/get/prefixes/{book_type}', [IssueBooksController::class, 'get_prefixes']);
    Route::post('/reports/pending_books', [IssueBooksController::class, 'get_pending_books_report']);
    
    // ROUTE ROUTES
    Route::get('/route', [RouteController::class, 'index']); 
    Route::get('/datatable/route', [RouteController::class, 'datatable']); 
    Route::post('/route', [RouteController::class, 'store']);
    Route::get('/autocomplete/route/', [RouteController::class, 'autocomplete']);
    Route::get('/route/{pk}', [RouteController::class, 'show']);
    Route::put('/route/{pk}', [RouteController::class, 'update']);
    Route::delete('/route/{pk}', [RouteController::class, 'destroy']);
    Route::get('/check/unique/route', [RouteController::class, 'check_unique']);
    Route::get('/get/route/get_route_data', [RouteController::class, 'get_route_data']);
    Route::get('/autocomplete/route/vehicle', [RouteController::class, 'autocomplete_for_vehicle']);
    Route::get('/autocomplete/route/station', [RouteController::class, 'autocomplete_for_station']);
    Route::post('/reports/route', [RouteController::class, 'get_route_report']);

    // PRICE LIST ROUTES
    Route::get('/price_list', [PriceListController::class, 'index']);  
    Route::post('/price_list', [PriceListController::class, 'store']);
    Route::get('/price_list/{pk}', [PriceListController::class, 'show']);
    Route::put('/price_list', [PriceListController::class, 'update']);
    Route::delete('/price_list', [PriceListController::class, 'destroy']);
    Route::post('/check/unique/price_list', [PriceListController::class, 'check_unique']);
    Route::get('/get/rate/for/gr_desc', [PriceListController::class, 'fetch_rate_for_gr_desc']);  
    Route::post('/reports/price_list', [PriceListController::class, 'get_price_list_report']); 

    // LATE DEDUCTION ROUTES
    Route::get('/late_deduction', [LateDeductionController::class, 'index']);  
    Route::post('/late_deduction', [LateDeductionController::class, 'store']);
    Route::get('/late_deduction/{pk}', [LateDeductionController::class, 'show']);
    Route::put('/late_deduction', [LateDeductionController::class, 'update']);
    Route::delete('/late_deduction', [LateDeductionController::class, 'destroy']);
    Route::post('/check/unique/late_deduction', [LateDeductionController::class, 'check_unique']);
    Route::get('/get/rate/for/ls', [LateDeductionController::class, 'fetch_rate_for_ls']);  
    Route::post('/reports/late_deduction', [LateDeductionController::class, 'get_late_deduction_report']); 

    // MATERIAL ROUTES
    Route::get('/material', [MaterialController::class, 'index']); 
    Route::get('/datatable/material', [MaterialController::class, 'datatable']); 
    Route::post('/material', [MaterialController::class, 'store']);
    Route::get('/autocomplete/material/', [MaterialController::class, 'autocomplete']);
    Route::get('/material/{pk}', [MaterialController::class, 'show']);
    Route::put('/material/{pk}', [MaterialController::class, 'update']);
    Route::delete('/material/{pk}', [MaterialController::class, 'destroy']);
    Route::get('/check/unique/material', [MaterialController::class, 'check_unique']);

    // UNIT ROUTES
    Route::get('/unit', [UnitController::class, 'index']); 
    Route::post('/unit', [UnitController::class, 'store']);
    Route::get('/autocomplete/unit/', [UnitController::class, 'autocomplete']);
    Route::get('/unit/{pk}', [UnitController::class, 'show']);
    Route::put('/unit/{pk}', [UnitController::class, 'update']);
    Route::delete('/unit/{pk}', [UnitController::class, 'destroy']);
    Route::get('/check/unique/unit', [UnitController::class, 'check_unique']);

    // VEHICLE TYPE ROUTES
    Route::get('/vehicle_type', [VehicleTypeController::class, 'index']); 
    Route::post('/vehicle_type', [VehicleTypeController::class, 'store']);
    Route::get('/autocomplete/vehicle_type/', [VehicleTypeController::class, 'autocomplete']);
    Route::get('/vehicle_type/{pk}', [VehicleTypeController::class, 'show']);
    Route::put('/vehicle_type/{pk}', [VehicleTypeController::class, 'update']);
    Route::delete('/vehicle_type/{pk}', [VehicleTypeController::class, 'destroy']);
    Route::get('/check/unique/vehicle_type', [VehicleTypeController::class, 'check_unique']);

    // PACKAGE METHOD ROUTES
    Route::get('/package_method', [PackageMethodController::class, 'index']); 
    Route::post('/package_method', [PackageMethodController::class, 'store']);
    Route::get('/autocomplete/package_method/', [PackageMethodController::class, 'autocomplete']);
    Route::get('/package_method/{pk}', [PackageMethodController::class, 'show']);
    Route::put('/package_method/{pk}', [PackageMethodController::class, 'update']);
    Route::delete('/package_method/{pk}', [PackageMethodController::class, 'destroy']);
    Route::get('/check/unique/package_method', [PackageMethodController::class, 'check_unique']);
    
    // ~~~~~~~~~~~~~~~~~~~~ END MASTER ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~


    // ~~~~~~~~~~~~~~~~~~~~ START ITEMS ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // Category ROUTES
    Route::get('/category', [CategoryController::class, 'index']); 
    Route::get('/datatable/category', [CategoryController::class, 'datatable']); 
    Route::post('/category', [CategoryController::class, 'store']);
    Route::get('/autocomplete/category/', [CategoryController::class, 'autocomplete']);
    Route::get('/category/{pk}', [CategoryController::class, 'show']);
    Route::put('/category/{pk}', [CategoryController::class, 'update']);
    Route::delete('/category/{pk}', [CategoryController::class, 'destroy']);
    Route::get('/check/unique/category', [CategoryController::class, 'check_unique']);

    // Items ROUTES
    Route::get('/item', [ItemController::class, 'index']); 
    Route::get('/datatable/item', [ItemController::class, 'datatable']); 
    Route::post('/item', [ItemController::class, 'store']);
    Route::get('/autocomplete/item/', [ItemController::class, 'autocomplete']);
    Route::get('/item_menu/item/', [ItemController::class, 'item_menu']);
    Route::get('/item/{pk}', [ItemController::class, 'show']);
    Route::put('/item/{pk}', [ItemController::class, 'update']);
    Route::delete('/item/{pk}', [ItemController::class, 'destroy']);
    Route::get('/check/unique/item', [ItemController::class, 'check_unique']);

    // ~~~~~~~~~~~~~~~~~~~~ END ITEMS ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // ~~~~~~~~~~~~~~~~~~~~ START LEADS ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // LEAD ROUTES
    Route::get('/lead', [LeadController::class, 'index']); 
    Route::get('/datatable/lead', [LeadController::class, 'datatable']); 
    Route::post('/lead', [LeadController::class, 'store']);
    Route::get('/autocomplete/lead/', [LeadController::class, 'autocomplete']);
    Route::get('/autocomplete/lead_for_quotation/', [LeadController::class, 'lead_for_quotation']);
    Route::get('/lead/{pk}', [LeadController::class, 'show']);
    Route::put('/lead/{pk}', [LeadController::class, 'update']);
    Route::delete('/lead/{pk}', [LeadController::class, 'destroy']);
    Route::get('/check/unique/lead', [LeadController::class, 'check_unique']);
    Route::get('/get/next/lead_number', [LeadController::class, 'get_next_lead_number']);
    Route::post('/assign_user/lead', [LeadController::class, 'assign_user_to_lead']);
    Route::post('/assign_surveyor/lead', [LeadController::class, 'assign_surveyor_to_lead']);
    Route::get('/get/lead_info/quotation/{pk}', [LeadController::class, 'get_lead_info_for_quotation']);
    

    // QUOTATION ROUTES
    Route::get('/quotation', [QuotationController::class, 'index']); 
    Route::get('/datatable/quotation', [QuotationController::class, 'datatable']); 
    Route::post('/quotation', [QuotationController::class, 'store']);
    Route::get('/autocomplete/quotation/', [QuotationController::class, 'autocomplete']);
    Route::get('/quotation/{pk}', [QuotationController::class, 'show']);
    Route::put('/quotation/{pk}', [QuotationController::class, 'update']);
    Route::delete('/quotation/{pk}', [QuotationController::class, 'destroy']);
    Route::get('/check/unique/quotation', [QuotationController::class, 'check_unique']);
    Route::get('/get/next/quotation_number', [QuotationController::class, 'get_next_quotation_number']);
    Route::get('/print/quotation/{pk}', [QuotationController::class, 'get_print_data']);

    // ~~~~~~~~~~~~~~~~~~~~ END LEADS ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // ~~~~~~~~~~~~~~~~~~~~ START HR ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // ATTENDANCE ROUTES
    Route::get('/get_month_attendance', [AttendanceController::class, 'get_month_attendance']);
    Route::post('/mark_attendance', [AttendanceController::class, 'mark_attendance']);
    Route::post('/mark_attendance', [AttendanceController::class, 'mark_attendance']);
    Route::post('/mark_attendance_for_all', [AttendanceController::class, 'mark_attendance_for_all']);

    // HOLIDAY ROUTES
    Route::get('/holiday', [HolidayController::class, 'index']); 
    Route::post('/holiday', [HolidayController::class, 'store']);
    Route::get('/autocomplete/holiday/', [HolidayController::class, 'autocomplete']);
    Route::get('/holiday/{pk}', [HolidayController::class, 'show']);
    Route::put('/holiday/{pk}', [HolidayController::class, 'update']);
    Route::delete('/holiday/{pk}', [HolidayController::class, 'destroy']);
    Route::get('/check/unique/holiday', [HolidayController::class, 'check_unique']);

    // SALARY ROUTES
    Route::get('/salary', [ SalaryController::class, 'index']); 
    Route::get('/datatable/salary', [ SalaryController::class, 'datatable']); 
    Route::post('/salary', [ SalaryController::class, 'store']);
    Route::get('/autocomplete/salary/', [ SalaryController::class, 'autocomplete']);
    Route::get('/salary/{pk}', [ SalaryController::class, 'show']);
    Route::put('/salary/{pk}', [ SalaryController::class, 'update']);
    Route::delete('/salary/{pk}', [ SalaryController::class, 'destroy']);
    Route::get('/check/unique/salary', [ SalaryController::class, 'check_unique']);
    Route::get('/get/recent/salary', [ SalaryController::class, 'get_recent']);
    Route::get('/get/next/salary_number', [ SalaryController::class, 'get_next_salary_number']);  
    Route::get('/get/custom_fields/salary', [ SalaryController::class, 'get_custom_fields']);  
    Route::get('/get/desc/for/salary', [ SalaryController::class, 'fetch_desc_for_salary']); 


    // ~~~~~~~~~~~~~~~~~~~~ END HR ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // ~~~~~~~~~~~~~~~~~~~~ START BOOKING ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // GR ROUTES
    Route::get('/gr', [GrController::class, 'index']); 
    Route::get('/datatable/gr', [GrController::class, 'datatable']); 
    Route::post('/gr', [GrController::class, 'store']);
    Route::get('/autocomplete/gr/', [GrController::class, 'autocomplete']);
    Route::get('/autocomplete/gr_for_pod/', [GrController::class, 'autocomplete_gr_for_pod']);
    Route::get('/gr/{pk}', [GrController::class, 'show']);
    Route::put('/gr/{pk}', [GrController::class, 'update']);
    Route::delete('/gr/{pk}', [GrController::class, 'destroy']);
    Route::get('/check/unique/gr', [GrController::class, 'check_unique']);
    Route::get('/get/recent/grs', [GrController::class, 'get_recent']);
    Route::get('/get/next/gr_number', [GrController::class, 'get_next_gr_number']);
    Route::get('/get/gr_descs/for/challan', [GrController::class, 'fetch_gr_desc_for_challan']);
    Route::get('/get/gr_details/for/pod/{pk}', [GrController::class, 'fetch_gr_details_for_pod']);    
    Route::get('/autocomplete/gr/for/challan', [GrController::class, 'autocomplete_gr_for_challan']);
    Route::post('/reports/gr', [GrController::class, 'get_gr_report']);     
    Route::post('/reports/gr_stock', [GrController::class, 'get_gr_stock_report']);  
    Route::put('/cancel/gr/{pk}', [GrController::class, 'cancel_gr']);
    Route::get('/reports/track_gr/{pk}', [GrController::class, 'track_gr']);
    Route::get('/autocomplete/gr/for/eway', [GrController::class, 'autocomplete_gr_for_eway']);
    Route::get('/eway/get_gr_data_for_eway_partb/{pk}', [GrController::class, 'get_gr_data_for_eway_partb']);
    Route::get('/check/issued/gr', [GrController::class, 'check_issued_gr']);

    // ~~~~~~~~~~~~~~~~~~~~ END BOOKING ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // ~~~~~~~~~~~~~~~~~~~~ START GOODS MOVE ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // CHALLAN ROUTES
    Route::get('/challan', [ChallanController::class, 'index']); 
    Route::get('/datatable/challan', [ChallanController::class, 'datatable']); 
    Route::post('/challan', [ChallanController::class, 'store']);
    Route::get('/autocomplete/challan/', [ChallanController::class, 'autocomplete']);
    Route::get('/autocomplete/challan_for_challan_receipt/', [ChallanController::class, 'challan_for_challan_receipt']);
    Route::get('/challan/{pk}', [ChallanController::class, 'show']);
    Route::put('/challan/{pk}', [ChallanController::class, 'update']);
    Route::delete('/challan/{pk}', [ChallanController::class, 'destroy']);
    Route::get('/check/unique/challan', [ChallanController::class, 'check_unique']);
    Route::get('/get/recent/challans', [ChallanController::class, 'get_recent']);
    Route::get('/get/next/challan_number', [ChallanController::class, 'get_next_challan_number']);  
    Route::get('/get/custom_fields/challan', [ChallanController::class, 'get_custom_fields']);  
    Route::get('/get/challan_descs/for/challan_receipt', [ChallanController::class, 'fetch_challan_desc_for_challan_receipt']); 
    Route::post('/reports/challan', [ChallanController::class, 'get_challan_report']);       

    // CHALLAN RECEIPT ROUTES
    Route::get('/challan_receipt', [ChallanReceiptController::class, 'index']); 
    Route::get('/datatable/challan_receipt', [ChallanReceiptController::class, 'datatable']); 
    Route::post('/challan_receipt', [ChallanReceiptController::class, 'store']);
    Route::get('/autocomplete/challan_receipt/', [ChallanReceiptController::class, 'autocomplete']);
    Route::get('/challan_receipt/{pk}', [ChallanReceiptController::class, 'show']);
    Route::put('/challan_receipt/{pk}', [ChallanReceiptController::class, 'update']);
    Route::delete('/challan_receipt/{pk}', [ChallanReceiptController::class, 'destroy']);
    Route::get('/check/unique/challan_receipt', [ChallanReceiptController::class, 'check_unique']);
    Route::get('/get/recent/challan_receipts', [ChallanReceiptController::class, 'get_recent']);
    Route::get('/get/next/challan_rec_number', [ChallanReceiptController::class, 'get_next_challan_rec_number']); 
    Route::post('/reports/challan_receipt', [ChallanReceiptController::class, 'get_challan_receipt_report']);    

    // UNLOADING ROUTES
    Route::get('/unloading', [UnloadingController::class, 'index']); 
    Route::get('/datatable/unloading', [UnloadingController::class, 'datatable']); 
    Route::post('/unloading', [UnloadingController::class, 'store']);
    Route::get('/autocomplete/unloading/', [UnloadingController::class, 'autocomplete']);
    Route::get('/unloading/{pk}', [UnloadingController::class, 'show']);
    Route::put('/unloading/{pk}', [UnloadingController::class, 'update']);
    Route::delete('/unloading/{pk}', [UnloadingController::class, 'destroy']);
    Route::get('/check/unique/unloading', [UnloadingController::class, 'check_unique']);
    Route::get('/get/recent/unloadings', [UnloadingController::class, 'get_recent']);
    Route::get('/get/next/unloading_number', [UnloadingController::class, 'get_next_unloading_number']); 

    // ~~~~~~~~~~~~~~~~~~~~ END  GOODS MOVE ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // ~~~~~~~~~~~~~~~~~~~~ START POD ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    // POD Routes
    Route::get('/pod', [PodController::class, 'index']); 
    Route::get('/datatable/pod', [PodController::class, 'datatable']); 
    Route::post('/pod', [PodController::class, 'store']);
    Route::get('/autocomplete/pod/', [PodController::class, 'autocomplete']);
    Route::get('/pod/{pk}', [PodController::class, 'show']);
    Route::put('/pod/{pk}', [PodController::class, 'update']);
    Route::delete('/pod/{pk}', [PodController::class, 'destroy']);
    Route::get('/check/unique/pod', [PodController::class, 'check_unique']);
    Route::get('/get/recent/pods', [PodController::class, 'get_recent']);
    Route::get('/get/next/pod_number', [PodController::class, 'get_next_pod_number']); 
    Route::post('/reports/pod', [PodController::class, 'get_pod_report']);
    Route::post('/reports/pending_pod', [PodController::class, 'get_pending_pod_report']);

    // ~~~~~~~~~~~~~~~~~~~~ END POD ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~


    // ~~~~~~~~~~~~~~~~~~~~ START FLEET OWNER ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    // Loading Slip Routes
    Route::get('/loading_slip', [LoadingSlipController::class, 'index']); 
    Route::get('/datatable/loading_slip', [LoadingSlipController::class, 'datatable']); 
    Route::post('/loading_slip', [LoadingSlipController::class, 'store']);
    Route::get('/autocomplete/loading_slip/', [LoadingSlipController::class, 'autocomplete']);
    Route::get('/autocomplete/loading_slip/linked', [LoadingSlipController::class, 'autocomplete_for_linked_ls']);
    Route::get('/loading_slip/{pk}', [LoadingSlipController::class, 'show']);
    Route::put('/loading_slip/{pk}', [LoadingSlipController::class, 'update']);
    Route::delete('/loading_slip/{pk}', [LoadingSlipController::class, 'destroy']);
    Route::get('/check/unique/loading_slip', [LoadingSlipController::class, 'check_unique']);
    Route::get('/get/recent/loading_slips', [LoadingSlipController::class, 'get_recent']);
    Route::get('/get/next/ls_number', [LoadingSlipController::class, 'get_next_ls_number']); 
    Route::post('/reports/loading_slip', [LoadingSlipController::class, 'get_loading_slip_report']); 
    Route::get('/check/fuel/loading_slip', [LoadingSlipController::class, 'check_fuel_in_master']);
    

    // Lorry Hire ROUTES
    Route::get('/lorry_hire', [LorryHireController::class, 'index']); 
    Route::get('/datatable/lorry_hire', [LorryHireController::class, 'datatable']); 
    Route::post('/lorry_hire', [LorryHireController::class, 'store']);
    Route::get('/autocomplete/lorry_hire/', [LorryHireController::class, 'autocomplete']);
    Route::get('/autocomplete/against_for_lorry_hire/', [LorryHireController::class, 'against_for_lorry_hire']);
    Route::get('/lorry_hire/{pk}', [LorryHireController::class, 'show']);
    Route::put('/lorry_hire/{pk}', [LorryHireController::class, 'update']);
    Route::delete('/lorry_hire/{pk}', [LorryHireController::class, 'destroy']);
    Route::get('/check/unique/lorry_hire', [LorryHireController::class, 'check_unique']);
    Route::get('/get/recent/lorry_hires', [LorryHireController::class, 'get_recent']);
    Route::get('/get/next/lh_number', [LorryHireController::class, 'get_next_lh_number']);  
    Route::get('/get/custom_fields/lorry_hire', [LorryHireController::class, 'get_custom_fields']);  
    Route::get('/get/against_desc/for/lorry_hire', [LorryHireController::class, 'fetch_against_details']); 
    Route::post('/reports/lorry_hire', [LorryHireController::class, 'get_lorry_hire_report']); 
    
    // DIESEL RECEIPT Routes
    Route::get('/diesel_receipt', [DieselReceiptController::class, 'index']); 
    Route::get('/datatable/diesel_receipt', [DieselReceiptController::class, 'datatable']); 
    Route::post('/diesel_receipt', [DieselReceiptController::class, 'store']);
    Route::get('/autocomplete/diesel_receipt/', [DieselReceiptController::class, 'autocomplete']);
    Route::get('/autocomplete/against_for_diesel_receipt/', [DieselReceiptController::class, 'against_for_diesel_receipt']);
    Route::get('/diesel_receipt/{pk}', [DieselReceiptController::class, 'show']);
    Route::put('/diesel_receipt/{pk}', [DieselReceiptController::class, 'update']);
    Route::delete('/diesel_receipt/{pk}', [DieselReceiptController::class, 'destroy']);
    Route::get('/check/unique/diesel_receipt', [DieselReceiptController::class, 'check_unique']);
    Route::get('/get/recent/diesel_receipts', [DieselReceiptController::class, 'get_recent']);
    Route::get('/get/next/dr_number', [DieselReceiptController::class, 'get_next_dr_number']); 
    Route::post('/reports/diesel_receipt', [DieselReceiptController::class, 'get_diesel_receipt_report']);


    // DRIVER SETTLEMENT ROUTES
    Route::get('/driver_settlement', [DriverSettlementController::class, 'index']); 
    Route::get('/datatable/driver_settlement', [DriverSettlementController::class, 'datatable']); 
    Route::post('/driver_settlement', [DriverSettlementController::class, 'store']);
    Route::get('/autocomplete/driver_settlement/', [DriverSettlementController::class, 'autocomplete']);
    Route::get('/driver_settlement/{pk}', [DriverSettlementController::class, 'show']);
    Route::put('/driver_settlement/{pk}', [DriverSettlementController::class, 'update']);
    Route::delete('/driver_settlement/{pk}', [DriverSettlementController::class, 'destroy']);
    Route::get('/check/unique/driver_settlement', [DriverSettlementController::class, 'check_unique']);
    Route::get('/get/recent/driver_settlements', [DriverSettlementController::class, 'get_recent']);
    Route::get('/get/next/ds_number', [DriverSettlementController::class, 'get_next_ds_number']);  
    Route::get('/get/custom_fields/driver_settlement', [DriverSettlementController::class, 'get_custom_fields']);  
    Route::get('/get/against_desc/for/driver_settlement', [DriverSettlementController::class, 'fetch_against_details']); 
    Route::post('/reports/driver_settlement', [DriverSettlementController::class, 'get_driver_settlement_report']); 
    Route::get('/autocomplete/vehicle_for_driver_settlement/', [DriverSettlementController::class, 'autocomplete_vehicle_for_driver_settlement']);

    // ~~~~~~~~~~~~~~~~~~~~ END FLEET OWNER ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~
    

    // ~~~~~~~~~~~~~~~~~~~~ START BILL ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    // BILL Routes
    Route::get('/bill', [BillController::class, 'index']); 
    Route::get('/datatable/bill', [BillController::class, 'datatable']); 
    Route::post('/bill', [BillController::class, 'store']);
    Route::get('/autocomplete/bill/', [BillController::class, 'autocomplete']);
    Route::get('/bill/{pk}', [BillController::class, 'show']);
    Route::put('/bill/{pk}', [BillController::class, 'update']);
    Route::delete('/bill/{pk}', [BillController::class, 'destroy']);
    Route::get('/check/unique/bill', [BillController::class, 'check_unique']);
    Route::get('/get/recent/bills', [BillController::class, 'get_recent']);
    Route::get('/get/next/bill_number', [BillController::class, 'get_next_bill_number']); 
    Route::get('/get/pod_and_gr_details/for/bill', [BillController::class, 'fetch_gr_and_pod_details_for_bill']); 
    Route::post('/reports/bill', [BillController::class, 'get_bill_report']);   
    Route::post('/reports/pending_bill_report', [BillController::class, 'get_pending_bill_report']);  
    Route::post('/reports/outstanding_bill', [BillController::class, 'get_outstanding_bill_report']); 
    Route::post('/reports/outstanding_creditors_bill', [BillController::class, 'get_outstanding_creditors_bill_report']);
    Route::post('/reports/gst', [BillController::class, 'get_gst_report']);  
    

    // CREDIT NOTE ROUTES
    Route::get('/credit_note', [CreditNoteController::class, 'index']); 
    Route::get('/datatable/credit_note', [CreditNoteController::class, 'datatable']); 
    Route::post('/credit_note', [CreditNoteController::class, 'store']);
    Route::get('/autocomplete/credit_note/', [CreditNoteController::class, 'autocomplete']);
    Route::get('/autocomplete/bill_for_credit_note/', [CreditNoteController::class, 'bill_for_credit_note']);
    Route::get('/credit_note/{pk}', [CreditNoteController::class, 'show']);
    Route::put('/credit_note/{pk}', [CreditNoteController::class, 'update']);
    Route::delete('/credit_note/{pk}', [CreditNoteController::class, 'destroy']);
    Route::get('/check/unique/credit_note', [CreditNoteController::class, 'check_unique']);
    Route::get('/get/recent/credit_notes', [CreditNoteController::class, 'get_recent']);
    Route::get('/get/next/cn_number', [CreditNoteController::class, 'get_next_cn_number']); 
    Route::get('/get/bill_details/for/credit_note', [CreditNoteController::class, 'fetch_bill_details_for_credit_note']); 

    // COVER LETTER ROUTES
    Route::get('/cover_letter', [CoverLetterController::class, 'index']); 
    Route::get('/datatable/cover_letter', [CoverLetterController::class, 'datatable']); 
    Route::post('/cover_letter', [CoverLetterController::class, 'store']);
    Route::get('/autocomplete/cover_letter/', [CoverLetterController::class, 'autocomplete']);
    Route::get('/cover_letter/{pk}', [CoverLetterController::class, 'show']);
    Route::put('/cover_letter/{pk}', [CoverLetterController::class, 'update']);
    Route::delete('/cover_letter/{pk}', [CoverLetterController::class, 'destroy']);
    Route::get('/check/unique/cover_letter', [CoverLetterController::class, 'check_unique']);
    Route::get('/get/recent/cover_letters', [CoverLetterController::class, 'get_recent']);
    Route::get('/get/next/cl_number', [CoverLetterController::class, 'get_next_cl_number']);  
    Route::get('/get/custom_fields/cover_letter', [CoverLetterController::class, 'get_custom_fields']);  
    Route::get('/get/bill_details/for/cover_letter', [CoverLetterController::class, 'fetch_against_details']); 

    // ~~~~~~~~~~~~~~~~~~~~ END BILL ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // ~~~~~~~~~~~~~~~~~~~~ START INVENTORY ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~

    // BRAND ROUTES
    Route::get('/brand', [BrandController::class, 'index']); 
    Route::get('/datatable/brand', [BrandController::class, 'datatable']); 
    Route::post('/brand', [BrandController::class, 'store']);
    Route::get('/autocomplete/brand/', [BrandController::class, 'autocomplete']);
    Route::get('/brand/{pk}', [BrandController::class, 'show']);
    Route::delete('/brand/{pk}', [BrandController::class, 'destroy']);
    Route::get('/check/unique/brand', [BrandController::class, 'check_unique']);

    // PRODUCT CATEGORY
    Route::get('/product_category', [ProductCategoryController::class, 'index']); 
    Route::get('/datatable/product_category', [ProductCategoryController::class, 'datatable']); 
    Route::post('/product_category', [ProductCategoryController::class, 'store']);
    Route::get('/autocomplete/product_category/', [ProductCategoryController::class, 'autocomplete']);
    Route::get('/product_category/{pk}', [ProductCategoryController::class, 'show']);
    Route::put('/product_category/{pk}', [ProductCategoryController::class, 'update']);
    Route::delete('/product_category/{pk}', [ProductCategoryController::class, 'destroy']);
    Route::get('/check/unique/product_category', [ProductCategoryController::class, 'check_unique']);

    // PRODUCT ROUTES
    Route::post('/product/update_openings', [ProductController::class, 'update_openings']);
    Route::post('/product/update_balance', [ProductController::class, 'update_balance_of_inventory']);
    Route::post('/product/balance_qty', [ProductController::class, 'calculate_balance_for_inventory']);
    Route::get('/product', [ProductController::class, 'index']); 
    Route::get('/datatable/product', [ProductController::class, 'datatable']); 
    Route::post('/product', [ProductController::class, 'store']);
    Route::get('/autocomplete/product/', [ProductController::class, 'autocomplete']);
    Route::get('/product/{pk}', [ProductController::class, 'show']);
    Route::put('/product/{pk}', [ProductController::class, 'update']);
    Route::delete('/product/{pk}', [ProductController::class, 'destroy']);
    Route::get('/check/unique/product', [ProductController::class, 'check_unique']);
    Route::post('/reports/inventory_report', [ProductController::class, 'get_inventory_report']);  

    // BILLS Routes
    Route::get('/inventory/bills', [PurchaseController::class, 'index']); 
    Route::get('/datatable/inventory/bills', [PurchaseController::class, 'datatable']); 
    Route::post('/inventory/bills', [PurchaseController::class, 'store']);
    Route::get('/autocomplete/inventory/bills/', [PurchaseController::class, 'autocomplete']);
    Route::get('/inventory/bills/{pk}', [PurchaseController::class, 'show']);
    Route::put('/inventory/bills/{pk}', [PurchaseController::class, 'update']);
    Route::delete('/inventory/bills/{pk}', [PurchaseController::class, 'destroy']);
    Route::get('/check/unique/inventory/bills', [PurchaseController::class, 'check_unique']);
    Route::get('/get/recent/inventory/bills', [PurchaseController::class, 'get_recent']);
    Route::get('/get/next/inventory/bill_number', [PurchaseController::class, 'get_next_bill_number']);
    Route::post('/reports/inventory/bills', [PurchaseController::class, 'get_inventory_bill_report']);
    Route::post('/reports/inventory/issue_stock', [PurchaseController::class, 'get_inventory_issue_report']);

    // ~~~~~~~~~~~~~~~~~~~~ END INVENTORY ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~


    // ~~~~~~~~~~~~~~~~~~~~ START REPORT ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~
    // ACCOUNTS REPORTS
    Route::post('/accounts/update_openings', [AccountController::class, 'update_openings']);
    Route::post('/accounts/update_balance', [AccountController::class, 'update_balance_of_account']);
    Route::get('/accounts/ledger/{pk}', [AccountController::class, 'get_ledger']);
    Route::get('/accounts/trial_balance', [AccountController::class, 'get_trial_balance']);
    Route::get('/accounts/truck_ledger_balance', [AccountController::class, 'get_truck_ledger_balance']);
    Route::get('/accounts/truck_pl_ledger/{pk}', [AccountController::class, 'get_truck_pl_ledger']);
    Route::get('/accounts/get_expanded_group/{pk}', [AccountController::class, 'get_expanded_group']);
    Route::get('/accounts/get_balance_for_account/{pk}', [AccountController::class, 'get_balance_for_account']);
    Route::get('/accounts/profit_and_loss', [AccountController::class, 'get_profit_and_loss']);
    Route::get('/accounts/balance_sheet', [AccountController::class, 'get_balance_sheet']);
    

    // ~~~~~~~~~~~~~~~~~~~~ END REPORT ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~


    
    // ~~~~~~~~~~~~~~~~~~~~ START SETTINGS ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~
    Route::get('/settings/fetchforselect', [SettingsController::class, 'fetch_for_select']);
    
    Route::get('/settings/dynamic_fields', [SettingsController::class, 'get_dynamic_fields']);
    Route::get('/settings/dynamic_fields/{name}', [SettingsController::class, 'get_single_dynamic_fields']);
    Route::put('/settings/dynamic_fields/{name}', [SettingsController::class, 'update_dynamic_fields']);

    Route::get('/settings/calculation_strings', [SettingsController::class, 'get_calculation_strings']);
    Route::get('/settings/calculation_strings/{name}', [SettingsController::class, 'get_single_calculation_strings']);
    Route::put('/settings/calculation_strings/{name}', [SettingsController::class, 'update_calculation_strings']);

    Route::get('/settings/general_settings', [SettingsController::class, 'get_general_settings']);
    Route::get('/settings/general_settings/{name}', [SettingsController::class, 'get_single_general_setting']);
    Route::put('/settings/general_settings/{name}', [SettingsController::class, 'update_general_settings']);
    
    // ~~~~~~~~~~~~~~~~~~~~ END SETTINGS ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~


    // ~~~~~~~~~~~~~~~~~~~~ START DASHBOARD ROUTES ~~~~~~~~~~~~~~~~~~~~~~~~~~
    Route::get('/dashboard', [DashboardController::class, 'index']);
    Route::get('/get/eway/reminders', [DashboardController::class, 'get_eway_reminders']);

    
    // THIRD PARTY APIS
    Route::get('/eway/get_eway_bill_data', [EwayController::class, 'getEwayBillData']);
    Route::get('/eway/create_consolidate', [EwayController::class, 'createConsolidate']);
    Route::get('/eway/updateAllValidity', [EwayController::class, 'updateAllValidity']);
    Route::get('/eway/createPartB', [EwayController::class, 'createPartB']);
    Route::get('/eway/getEwayNo', [EwayController::class, 'getEwayNo']);

    Route::get('/e-invoice/cancel', [EinvoiceController::class, 'cancel_einvoice']);
    Route::get('/e-invoice/generate', [EinvoiceController::class, 'generate_einvoice']);

    Route::get('/e-invoice/credit_debit_note/cancel', [EinvoiceController::class, 'cancel_einvoice']);
    Route::get('/e-invoice/credit_debit_note/generate', [EinvoiceController::class, 'generate_credit_debit_note']);
    

    Route::get('/get_details_from_pincode/{pincode}', function ($pincode) {
        try {
            $response = Http::get("https://www.postpincode.in/api/getCityName.php?pincode=$pincode");
            
            if ($response->successful()) {
                return $response->body();
            } else {
                // Handle non-successful responses (e.g., 404 Not Found)
                return response()->json(['error' => 'Failed to fetch pincode details'], $response->status());
            }
        } catch (\Exception $e) {
            // Handle exceptions (e.g., connection error)
            return response()->json(['error' => 'An error occurred while fetching data'], 500);
        }
    });
});






